@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -152,12 +152,12 @@ generate_main(int num)
 static char *
 usage(char *s)
 {
-  j_printerr("generate --- sentence random generator\n",s);
-  j_printerr("usage: %s [-v] [-n] prefix\n",s);
-  j_printerr("  -n num  ... generate N sentences (default: 10)\n");
-  j_printerr("  -t      ... use category symbols instead of words (needs .term)\n");
-  j_printerr("  -s string ... specify short-pause model\n");
-  j_printerr("  -v      ... verbose output\n");
+  fprintf(stderr, "generate --- sentence random generator\n");
+  fprintf(stderr, "usage: %s [-v] [-n] prefix\n",s);
+  fprintf(stderr, "  -n num  ... generate N sentences (default: 10)\n");
+  fprintf(stderr, "  -t      ... use category symbols instead of words (needs .term)\n");
+  fprintf(stderr, "  -s string ... specify short-pause model\n");
+  fprintf(stderr, "  -v      ... verbose output\n");
   exit(1);
 }
 
@@ -201,7 +201,7 @@ int main(int argc, char *argv[])
 	spname = argv[i];
 	break;
       default:
-	j_printerr("no such option: %s\n",argv[i]);
+	fprintf(stderr, "no such option: %s\n",argv[i]);
 	usage(argv[0]);
       }
     } else {
