@@ -95,7 +95,7 @@
  * @author Akinobu LEE
  * @date   Sat Feb 12 13:20:53 2005
  *
- * $Revision: 1.4 $
+ * $Revision: 1.7 $
  * 
  */
 /*
@@ -134,12 +134,16 @@
  * @callergraph
  * @callgraph
  */
-void
+boolean
 adin_setup_param(ADIn *adin, Jconf *jconf)
 {
   float samples_in_msec;
   int freq;
 
+  if (jconf->input.sfreq <= 0) {
+    jlog("ERROR: adin_setup_param: going to set smpfreq to %d\n", jconf->input.sfreq);
+    return FALSE;
+  }
   if (jconf->detect.silence_cut < 2) {
     adin->adin_cut_on = (jconf->detect.silence_cut == 1) ? TRUE : FALSE;
   } else {
@@ -190,6 +194,8 @@ adin_setup_param(ADIn *adin, Jconf *jconf)
 
   adin->rehash = FALSE;
 
+  return TRUE;
+
 }
 
 /** 
@@ -334,11 +340,7 @@ adin_cut(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Recog *), Reco
   }
 
   if (!a->adin_cut_on && a->is_valid_data == TRUE) {
-#ifdef HAVE_PTHREAD
-    if (!a->enable_thread) callback_exec(CALLBACK_EVENT_SPEECH_START, recog);
-#else
     callback_exec(CALLBACK_EVENT_SPEECH_START, recog);
-#endif
   }
 
   /*************/
@@ -469,11 +471,7 @@ adin_cut(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Recog *), Reco
        TRUE here. */ 
     if (!a->adin_cut_on && a->is_valid_data == FALSE && a->current_len > 0) {
       a->is_valid_data = TRUE;
-#ifdef HAVE_PTHREAD
-      if (!a->enable_thread) callback_exec(CALLBACK_EVENT_SPEECH_START, recog);
-#else
       callback_exec(CALLBACK_EVENT_SPEECH_START, recog);
-#endif
     }
 
     /******************************************************/
@@ -547,11 +545,7 @@ adin_cut(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Recog *), Reco
 #ifdef THREAD_DEBUG
 	    jlog("DEBUG: detect on\n");
 #endif
-#ifdef HAVE_PTHREAD
-	    if (!a->enable_thread) callback_exec(CALLBACK_EVENT_SPEECH_START, recog);
-#else
 	    callback_exec(CALLBACK_EVENT_SPEECH_START, recog);
-#endif
 
 	    /****************************************/
 	    /* flush samples stored in cycle buffer */
@@ -577,11 +571,7 @@ adin_cut(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Recog *), Reco
 #ifdef THREAD_DEBUG
 		jlog("DEBUG: callback for buffered samples (%d bytes)\n", len - wstep);
 #endif
-#ifdef HAVE_PTHREAD
-		if (!a->enable_thread) callback_exec_adin(CALLBACK_ADIN_TRIGGERED, recog, a->cbuf, len - wstep);
-#else
 		callback_exec_adin(CALLBACK_ADIN_TRIGGERED, recog, a->cbuf, len - wstep);
-#endif
 		ad_process_ret = (*ad_process)(a->cbuf, len - wstep, recog);
 		switch(ad_process_ret) {
 		case 1:		/* segmentation notification from process callback */
@@ -651,11 +641,7 @@ adin_cut(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Recog *), Reco
 #ifdef THREAD_DEBUG
 		jlog("DEBUG: callback for swapped %d samples\n", a->sblen);
 #endif
-#ifdef HAVE_PTHREAD
-		if (!a->enable_thread) callback_exec_adin(CALLBACK_ADIN_TRIGGERED, recog, a->swapbuf, a->sblen);
-#else
 		callback_exec_adin(CALLBACK_ADIN_TRIGGERED, recog, a->swapbuf, a->sblen);
-#endif
 		ad_process_ret = (*ad_process)(a->swapbuf, a->sblen, recog);
 		a->sblen = 0;
 		switch(ad_process_ret) {
@@ -777,11 +763,7 @@ adin_cut(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Recog *), Reco
 	    jlog("DEBUG: callback for input sample [%d-%d]\n", i, i+wstep);
 #endif
 	    /* call external function */
-#ifdef HAVE_PTHREAD
-	    if (!a->enable_thread) callback_exec_adin(CALLBACK_ADIN_TRIGGERED, recog, &(a->buffer[i]), wstep);
-#else
 	    callback_exec_adin(CALLBACK_ADIN_TRIGGERED, recog, &(a->buffer[i]), wstep);
-#endif
 	    ad_process_ret = (*ad_process)(&(a->buffer[i]), wstep, recog);
 	    switch(ad_process_ret) {
 	    case 1:		/* segmentation notification from process callback */
@@ -825,6 +807,7 @@ adin_cut(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Recog *), Reco
 	/* end input by silence */
 	a->is_valid_data = FALSE;	/* turn off processing */
 	a->sblen = 0;
+	callback_exec(CALLBACK_EVENT_SPEECH_STOP, recog);
 #ifdef HAVE_PTHREAD
 	if (a->enable_thread) { /* just stop transfer */
 	  pthread_mutex_lock(&(a->mutex));
@@ -867,15 +850,9 @@ break_input:
       end_status = -1;
     }
   }
-
-  /* execute callback */
-#ifdef HAVE_PTHREAD
-  if (!a->enable_thread) callback_exec(CALLBACK_EVENT_SPEECH_STOP, recog);
-#else
-  callback_exec(CALLBACK_EVENT_SPEECH_STOP, recog);
-#endif
-
   if (a->end_of_stream) {			/* input already ends */
+    /* execute callback */
+    callback_exec(CALLBACK_EVENT_SPEECH_STOP, recog);
     if (a->bp == 0) {		/* rest buffer successfully flushed */
       /* reset status */
       a->need_init = TRUE;		/* bufer status shoule be reset at next call */
@@ -1032,7 +1009,6 @@ adin_thread_process(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Rec
   int i;
   boolean overflowed_p;
   boolean transfer_online_local;
-  boolean first_trig;
   ADIn *a;
 
   a = recog->adin;
@@ -1048,7 +1024,6 @@ adin_thread_process(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Rec
 
   /* main processing loop */
   prev_len = 0;
-  first_trig = TRUE;
   for(;;) {
     /* get current length (locking) */
     pthread_mutex_lock(&(a->mutex));
@@ -1065,7 +1040,6 @@ adin_thread_process(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Rec
       a->speechlen = 0;
       a->transfer_online = transfer_online_local = FALSE;
       pthread_mutex_unlock(&(a->mutex));
-      if (!first_trig) callback_exec(CALLBACK_EVENT_SPEECH_STOP, recog);
       return(1);		/* return with segmented status */
     }
     /* callback poll */
@@ -1076,7 +1050,6 @@ adin_thread_process(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Rec
 	  a->transfer_online = transfer_online_local = FALSE;
 	  a->speechlen = 0;
 	  pthread_mutex_unlock(&(a->mutex));
-	  if (!first_trig) callback_exec(CALLBACK_EVENT_SPEECH_STOP, recog);
 	  return(-2);
 	}
       }
@@ -1092,13 +1065,7 @@ adin_thread_process(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Rec
 	 So locking is not needed while processing.
        */
       /*jlog("DEBUG: main: read %d-%d\n", prev_len, nowlen);*/
-      /* call on/off callback */
-      if (first_trig) {
-	first_trig = FALSE;
-	callback_exec(CALLBACK_EVENT_SPEECH_START, recog);
-      }
       if (ad_process != NULL) {
-	callback_exec_adin(CALLBACK_ADIN_TRIGGERED, recog, &(a->speech[prev_len]), nowlen - prev_len);
 	ad_process_ret = (*ad_process)(&(a->speech[prev_len]), nowlen - prev_len, recog);
 #ifdef THREAD_DEBUG
 	jlog("DEBUG: ad_process_ret=%d\n", ad_process_ret);
@@ -1116,14 +1083,12 @@ adin_thread_process(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Rec
 	  }
 	  a->transfer_online = transfer_online_local = FALSE;
 	  pthread_mutex_unlock(&(a->mutex));
-	  if (!first_trig) callback_exec(CALLBACK_EVENT_SPEECH_STOP, recog);
 	  /* keep transfering */
 	  return(2);		/* return with segmented status */
 	case -1:		/* error */
 	  pthread_mutex_lock(&(a->mutex));
 	  a->transfer_online = transfer_online_local = FALSE;
 	  pthread_mutex_unlock(&(a->mutex));
-	  if (!first_trig) callback_exec(CALLBACK_EVENT_SPEECH_STOP, recog);
 	  return(-1);		/* return with error */
 	}
       }
@@ -1145,7 +1110,6 @@ adin_thread_process(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Rec
 	pthread_mutex_lock(&(a->mutex));
 	a->speechlen = 0;
 	pthread_mutex_unlock(&(a->mutex));
-	if (!first_trig) callback_exec(CALLBACK_EVENT_SPEECH_STOP, recog);
         break;
       }
       usleep(50000);   /* wait = 0.05sec*/            
@@ -1197,6 +1161,8 @@ adin_thread_process(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Rec
 int
 adin_go(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Recog *), Recog *recog)
 {
+  /* output listening start message */
+  callback_exec(CALLBACK_EVENT_SPEECH_READY, recog);
 #ifdef HAVE_PTHREAD
   if (recog->adin->enable_thread) {
     return(adin_thread_process(ad_process, ad_check, recog));
