@@ -18,7 +18,7 @@
  * @author Akinobu Lee
  * @date   Thu May 12 18:52:07 2005
  *
- * $Revision: 1.4 $
+ * $Revision: 1.8 $
  * 
  */
 /*
@@ -182,6 +182,10 @@ opt_parse(int argc, char *argv[], char *cwd, Jconf *jconf)
       continue;
     } else if (strmatch(argv[i],"-AM_GMM") || strmatch(argv[i], "[AM_GMM]")) {
       /* switch current to GMM */
+      if (jconf->gmm == NULL) {
+	/* if new, allocate jconf for GMM */
+	jconf->gmm = j_jconf_am_new();
+      }
       jconf->amnow = jconf->gmm;
       continue;
     } else if (strmatch(argv[i],"-LM") || strmatch(argv[i], "[LM]")) {
@@ -544,6 +548,9 @@ opt_parse(int argc, char *argv[], char *cwd, Jconf *jconf)
     } else if (strmatch(argv[i],"-debug")) { /* debug mode: output huge log */
       debug2_flag = verbose_flag = TRUE;
       continue;
+    } else if (strmatch(argv[i],"-callbackdebug")) { /* output callback debug message */
+      callback_debug_flag = TRUE;
+      continue;
     } else if (strmatch(argv[i],"-progout")) { /* enable progressive output */
       jconf->searchnow->output.progout_flag = TRUE;
       continue;
@@ -732,6 +739,12 @@ opt_parse(int argc, char *argv[], char *cwd, Jconf *jconf)
     } else if (strmatch(argv[i],"-nozmeanframe")) { /* disable frame-wise DC offset by zero mean */
       jconf->amnow->analysis.para.zmeanframe = FALSE;
       continue;
+    } else if (strmatch(argv[i],"-usepower")) { /* use power instead of magnitude in filterbank analysis */
+      jconf->amnow->analysis.para.usepower = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-nousepower")) { /* use magnitude in fbank analysis (default)  */
+      jconf->amnow->analysis.para.usepower = FALSE;
+      continue;
     } else if (strmatch(argv[i],"-spsegment")) { /* enable short-pause segmentation */
       jconf->searchnow->successive.enabled = TRUE;
       continue;
@@ -973,6 +986,9 @@ opt_parse(int argc, char *argv[], char *cwd, Jconf *jconf)
     } else if (strmatch(argv[i], "-active")) { /* start active (default) */
       jconf->searchnow->sw.start_inactive = FALSE;
       continue;
+    } else if (strmatch(argv[i],"-fallback1pass")) { /* use 1st pass result on search failure */
+      jconf->searchnow->sw.fallback_pass1_flag = TRUE;
+      continue;
     }
     if (argv[i][0] == '-' && strlen(argv[i]) == 2) {
       /* 1-letter options */
