@@ -12,7 +12,7 @@
  * @author Akinobu Lee
  * @date   Fri May 13 15:04:34 2005
  *
- * $Revision: 1.3 $
+ * $Revision: 1.6 $
  * 
  */
 /*
@@ -53,6 +53,8 @@ j_output_argument_help(FILE *fp)
   j_put_compile_defs(fp);
   fprintf(fp, "\nOptions:\n");
 
+  fprintf(fp, "\n--- Global Options -----------------------------------------------\n");
+
   fprintf(fp, "\n Speech Input:\n");
   fprintf(fp, "    (Can extract only MFCC based features from waveform)\n");
   fprintf(fp, "    [-input devname]    input source  (default = htkparam)\n");
@@ -72,11 +74,8 @@ j_output_argument_help(FILE *fp)
   fprintf(fp, "    [-NA host:unit]     get audio from NetAudio server at host:unit\n");
 #endif
   fprintf(fp, "    [-adport portnum]   adinnet port number to listen         (%d)\n", jconf->input.adinnet_port);
-  fprintf(fp, "    [-smpFreq freq]     sample period in Hz                   (%ld)\n", jconf->am_root->analysis.para_default.smp_freq);
-  fprintf(fp, "    [-smpPeriod period] sample period in 100ns unit           (%ld)\n", jconf->am_root->analysis.para_default.smp_period);
   fprintf(fp, "    [-48]               enable 48kHz sampling with internal down sampler (OFF)\n");
   fprintf(fp, "    [-zmean/-nozmean]   enable/disable DC offset removal      (OFF)\n");
-  fprintf(fp, "    [-zmeanframe/-nozmeanframe] frame-wise DC removal like HTK(OFF)\n");
   fprintf(fp, "    [-nostrip]          disable stripping off zero samples\n");
   fprintf(fp, "    [-record dir]       record triggered speech data to dir\n");
   fprintf(fp, "    [-rejectshort msec] reject an input shorter than specified\n");
@@ -94,6 +93,40 @@ j_output_argument_help(FILE *fp)
   fprintf(fp, "    [-headmargin msec]  header margin length in msec.         (%d)\n", jconf->detect.head_margin_msec);
   fprintf(fp, "    [-tailmargin msec]  tail margin length in msec.           (%d)\n", jconf->detect.tail_margin_msec);
 
+  fprintf(fp, "\n GMM utterance verification:\n");
+  fprintf(fp, "    -gmm filename       GMM definition file\n");
+  fprintf(fp, "    -gmmnum num         GMM Gaussian pruning num              (%d)\n", jconf->reject.gmm_gprune_num);
+  fprintf(fp, "    -gmmreject string   comma-separated list of noise model name to reject\n");
+#ifdef GMM_VAD
+  fprintf(fp, "\n GMM-based VAD:\n");
+  fprintf(fp, "    -gmmmargin frames   backstep margin on speech trigger     (%d)\n", jconf->detect.gmm_margin);
+  fprintf(fp, "    -gmmup score        up-trigger threshold                  (%.1f)\n", jconf->detect.gmm_uptrigger_thres);
+  fprintf(fp, "    -gmmdown score      down-trigger threshold                (%.1f)\n", jconf->detect.gmm_downtrigger_thres);
+#endif
+
+  fprintf(fp, "\n On-the-fly Decoding: (default: on=mic/net off=files)\n");
+  fprintf(fp, "    [-realtime]         turn on, input streamed with MAP-CMN\n");
+  fprintf(fp, "    [-norealtime]       turn off, input buffered with sentence CMN\n");
+
+  fprintf(fp, "\n Others:\n");
+  fprintf(fp, "    [-C jconffile]      load options from jconf file\n");
+  fprintf(fp, "    [-quiet]            reduce output to only word string\n");
+  fprintf(fp, "    [-demo]             equal to \"-quiet -progout\"\n");
+  fprintf(fp, "    [-debug]            (for debug) dump numerous log\n");
+  fprintf(fp, "    [-callbackdebug]    (for debug) output message per callback\n");
+  fprintf(fp, "    [-check (wchmm|trellis)] (for debug) check internal structure\n");
+  fprintf(fp, "    [-check triphone]   triphone mapping check\n");
+  fprintf(fp, "    [-setting]          print engine configuration and exit\n");
+  fprintf(fp, "    [-help]             print this message and exit\n");
+
+  fprintf(fp, "\n--- Instance Declarations ----------------------------------------\n\n");
+
+  fprintf(fp, "    [-AM]               start a new acoustic model instance\n");
+  fprintf(fp, "    [-LM]               start a new language model instance\n");
+  fprintf(fp, "    [-SR]               start a new recognizer (search) instance\n");
+  fprintf(fp, "    [-AM_GMM]           start an AM feature instance for GMM\n");
+  fprintf(fp, "\n--- Acoustic Model Options (-AM) ---------------------------------\n");
+
   fprintf(fp, "\n Acoustic analysis:\n");
   fprintf(fp, "    [-htkconf file]     load parameters from the HTK Config file\n");
   fprintf(fp, "    [-smpFreq freq]     sample period (Hz)                    (%ld)\n", jconf->am_root->analysis.para_default.smp_freq);
@@ -116,53 +149,13 @@ j_output_argument_help(FILE *fp)
   fprintf(fp, "    [-ssload filename]  load constant noise spectrum from file for SS\n");
   fprintf(fp, "    [-ssalpha value]    alpha coef. for SS                    (%f)\n", jconf->am_root->frontend.ss_alpha);
   fprintf(fp, "    [-ssfloor value]    spectral floor for SS                 (%f)\n", jconf->am_root->frontend.ss_floor);
+  fprintf(fp, "    [-zmeanframe/-nozmeanframe] frame-wise DC removal like HTK(OFF)\n");
+  fprintf(fp, "    [-usepower/-nousepower] use power in fbank analysis       (OFF)\n");
+  fprintf(fp, "    [-cmnload file]     load initial CMN param from file on startup\n");
+  fprintf(fp, "    [-cmnsave file]     save CMN param to file after each input\n");
+  fprintf(fp, "    [-cmnnoupdate]      not update CMN param while recog. (use with -cmnload)\n");
+  fprintf(fp, "    [-cmnmapweight]     weight value of initial cm for MAP-CMN (%6.2f)\n", jconf->am_root->analysis.cmn_map_weight);
 
-  fprintf(fp, "\n GMM utterance verification:\n");
-  fprintf(fp, "    -gmm filename       GMM definition file\n");
-  fprintf(fp, "    -gmmnum num         GMM Gaussian pruning num              (%d)\n", jconf->reject.gmm_gprune_num);
-  fprintf(fp, "    -gmmreject string   comma-separated list of noise model name to reject\n");
-#ifdef GMM_VAD
-  fprintf(fp, "\n GMM-based VAD:\n");
-  fprintf(fp, "    -gmmmargin frames   backstep margin on speech trigger     (%d)\n", jconf->detect.gmm_margin);
-  fprintf(fp, "    -gmmup score        up-trigger threshold                  (%.1f)\n", jconf->detect.gmm_uptrigger_thres);
-  fprintf(fp, "    -gmmdown score      down-trigger threshold                (%.1f)\n", jconf->detect.gmm_downtrigger_thres);
-#endif
-
-  fprintf(fp, "\n N-gram Language model:\n");
-  fprintf(fp, "    -d file.bingram     n-gram file in Julius binary format\n");
-  fprintf(fp, "    -nlr file.arpa      forward n-gram file in ARPA format\n");
-  fprintf(fp, "    -nrl file.arpa      backward n-gram file in ARPA format\n");
-  fprintf(fp, "    [-lmp float float]  weight and penalty (tri: %.1f %.1f mono: %.1f %1.f)\n", DEFAULT_LM_WEIGHT_TRI_PASS1, DEFAULT_LM_PENALTY_TRI_PASS1, DEFAULT_LM_WEIGHT_MONO_PASS1, DEFAULT_LM_PENALTY_MONO_PASS1);
-  fprintf(fp, "    [-lmp2 float float]       for 2nd pass (tri: %.1f %.1f mono: %.1f %1.f)\n", DEFAULT_LM_WEIGHT_TRI_PASS2, DEFAULT_LM_PENALTY_TRI_PASS2, DEFAULT_LM_WEIGHT_MONO_PASS2, DEFAULT_LM_PENALTY_MONO_PASS2);
-  fprintf(fp, "    [-transp float]     penalty for transparent word (%+2.1f)\n", jconf->search_root->lmp.lm_penalty_trans);
-
-  fprintf(fp, "\n DFA Grammar:\n");
-  fprintf(fp, "    -dfa file.dfa       DFA grammar file\n");
-  fprintf(fp, "    -gram file[,file2...] (list of) grammar prefix(es)\n");
-  fprintf(fp, "    -gramlist filename  filename of grammar list\n");
-  fprintf(fp, "    [-penalty1 float]   word insertion penalty (1st pass)     (%.1f)\n", jconf->search_root->lmp.penalty1);
-  fprintf(fp, "    [-penalty2 float]   word insertion penalty (2nd pass)     (%.1f)\n", jconf->search_root->lmp.penalty2);
-
-  fprintf(fp, "\n Word Dictionary: (for N-gram and DFA)\n");
-  fprintf(fp, "    -v dictfile         dictionary file name\n");
-  fprintf(fp, "    [-silhead wordname] (n-gram) beginning-of-sentence word   (%s)\n", BEGIN_WORD_DEFAULT);
-  fprintf(fp, "    [-siltail wordname] (n-gram) end-of-sentence word         (%s)\n", END_WORD_DEFAULT);
-  fprintf(fp, "    [-forcedict]        ignore error entry and keep running\n");
-  fprintf(fp, "    [-iwspword]         (n-gram) add short-pause word for inter-word CD sp\n");
-  fprintf(fp, "    [-iwspentry entry]  (n-gram) word entry for \"-iwspword\" (%s)\n", IWSPENTRY_DEFAULT);
-  
-  fprintf(fp, "\n Isolated Word Recognition:\n");
-  fprintf(fp, "    -w file[,file2...]  (list of) wordlist file name(s)\n");
-  fprintf(fp, "    -wlist filename     file that contains list of wordlists\n");
-  fprintf(fp, "    -wsil head tail sp  name of silence/pause model\n");
-  fprintf(fp, "                          head - BOS silence model name       (%s)\n", jconf->lm_root->wordrecog_head_silence_model_name);
-  fprintf(fp, "                          tail - EOS silence model name       (%s)\n", jconf->lm_root->wordrecog_tail_silence_model_name);
-  fprintf(fp, "                           sp  - their name as context or \"NULL\" (%s)\n", (jconf->lm_root->wordrecog_silence_context_name[0] == '\0') ? "NULL" : jconf->lm_root->wordrecog_silence_context_name);
-#ifdef DETERMINE
-  fprintf(fp, "    -wed float int      thresholds for early word determination\n");
-  fprintf(fp, "                        float: score threshold    (%.1f)\n", jconf->search_root->pass1.determine_score_thres);
-  fprintf(fp, "                        int: frame duration thres (%d)\n", jconf->search_root->pass1.determine_duration_thres);
-#endif
   fprintf(fp, "\n Acoustic Model:\n");
   fprintf(fp, "    -h hmmdefsfile      HMM definition file name\n");
   fprintf(fp, "    [-hlist HMMlistfile] HMMlist filename (must for triphone model)\n");
@@ -176,7 +169,7 @@ j_output_argument_help(FILE *fp)
   fprintf(fp, "    [-spmodel HMMname]  name of short pause model             (\"%s\")\n", SPMODEL_NAME_DEFAULT);
   fprintf(fp, "    [-multipath]        switch decoding for multi-path HMM    (auto)\n");
 
-  fprintf(fp, "\n Acoustic Computation Method:\n");
+  fprintf(fp, "\n Acoustic Model Computation Method:\n");
   fprintf(fp, "    [-gprune methodname] select Gaussian pruning method:\n");
 #ifdef GPRUNE_DEFAULT_SAFE
   fprintf(fp, "             safe          safe pruning (default for TM/PTM)\n");
@@ -198,16 +191,48 @@ j_output_argument_help(FILE *fp)
   fprintf(fp, "    [-gshmm hmmdefs]    monophone hmmdefs for GS\n");
   fprintf(fp, "    [-gsnum N]          N-best state will be selected        (%d)\n", jconf->am_root->gs_statenum);
 
-  fprintf(fp, "\n Short-pause Segmentation:\n");
-  fprintf(fp, "    [-spsegment]        enable short-pause segmentation\n");
-  fprintf(fp, "    [-spdur]            length threshold of sp frames         (%d)\n", jconf->search_root->successive.sp_frame_duration);
-#ifdef SPSEGMENT_NAIST
-  fprintf(fp, "    [-spmargin]         backstep margin on speech trigger     (%d)\n", jconf->search_root->successive.sp_margin);
-  fprintf(fp, "    [-spdelay]          delay on speech trigger               (%d)\n", jconf->search_root->successive.sp_delay);
-#endif
-  fprintf(fp, "    [-pausemodels str]  comma-delimited list of pause models for segment\n");
+  fprintf(fp, "\n--- Language Model Options (-LM) ---------------------------------\n");
 
-  fprintf(fp, "\n Search Parameters (First Pass):\n");
+  fprintf(fp, "\n N-gram:\n");
+  fprintf(fp, "    -d file.bingram     n-gram file in Julius binary format\n");
+  fprintf(fp, "    -nlr file.arpa      forward n-gram file in ARPA format\n");
+  fprintf(fp, "    -nrl file.arpa      backward n-gram file in ARPA format\n");
+  fprintf(fp, "    [-lmp float float]  weight and penalty (tri: %.1f %.1f mono: %.1f %1.f)\n", DEFAULT_LM_WEIGHT_TRI_PASS1, DEFAULT_LM_PENALTY_TRI_PASS1, DEFAULT_LM_WEIGHT_MONO_PASS1, DEFAULT_LM_PENALTY_MONO_PASS1);
+  fprintf(fp, "    [-lmp2 float float]       for 2nd pass (tri: %.1f %.1f mono: %.1f %1.f)\n", DEFAULT_LM_WEIGHT_TRI_PASS2, DEFAULT_LM_PENALTY_TRI_PASS2, DEFAULT_LM_WEIGHT_MONO_PASS2, DEFAULT_LM_PENALTY_MONO_PASS2);
+  fprintf(fp, "    [-transp float]     penalty for transparent word (%+2.1f)\n", jconf->search_root->lmp.lm_penalty_trans);
+
+  fprintf(fp, "\n DFA Grammar:\n");
+  fprintf(fp, "    -dfa file.dfa       DFA grammar file\n");
+  fprintf(fp, "    -gram file[,file2...] (list of) grammar prefix(es)\n");
+  fprintf(fp, "    -gramlist filename  filename of grammar list\n");
+  fprintf(fp, "    [-penalty1 float]   word insertion penalty (1st pass)     (%.1f)\n", jconf->search_root->lmp.penalty1);
+  fprintf(fp, "    [-penalty2 float]   word insertion penalty (2nd pass)     (%.1f)\n", jconf->search_root->lmp.penalty2);
+
+  fprintf(fp, "\n Word Dictionary for N-gram and DFA:\n");
+  fprintf(fp, "    -v dictfile         dictionary file name\n");
+  fprintf(fp, "    [-silhead wordname] (n-gram) beginning-of-sentence word   (%s)\n", BEGIN_WORD_DEFAULT);
+  fprintf(fp, "    [-siltail wordname] (n-gram) end-of-sentence word         (%s)\n", END_WORD_DEFAULT);
+  fprintf(fp, "    [-forcedict]        ignore error entry and keep running\n");
+  fprintf(fp, "    [-iwspword]         (n-gram) add short-pause word for inter-word CD sp\n");
+  fprintf(fp, "    [-iwspentry entry]  (n-gram) word entry for \"-iwspword\" (%s)\n", IWSPENTRY_DEFAULT);
+  
+  fprintf(fp, "\n Isolated Word Recognition:\n");
+  fprintf(fp, "    -w file[,file2...]  (list of) wordlist file name(s)\n");
+  fprintf(fp, "    -wlist filename     file that contains list of wordlists\n");
+  fprintf(fp, "    -wsil head tail sp  name of silence/pause model\n");
+  fprintf(fp, "                          head - BOS silence model name       (%s)\n", jconf->lm_root->wordrecog_head_silence_model_name);
+  fprintf(fp, "                          tail - EOS silence model name       (%s)\n", jconf->lm_root->wordrecog_tail_silence_model_name);
+  fprintf(fp, "                           sp  - their name as context or \"NULL\" (%s)\n", (jconf->lm_root->wordrecog_silence_context_name[0] == '\0') ? "NULL" : jconf->lm_root->wordrecog_silence_context_name);
+#ifdef DETERMINE
+  fprintf(fp, "    -wed float int      thresholds for early word determination\n");
+  fprintf(fp, "                        float: score threshold    (%.1f)\n", jconf->search_root->pass1.determine_score_thres);
+  fprintf(fp, "                        int: frame duration thres (%d)\n", jconf->search_root->pass1.determine_duration_thres);
+#endif
+
+  fprintf(fp, "\n--- Recognizer / Search Options (-SR) ----------------------------\n");
+
+
+  fprintf(fp, "\n Search Parameters for the First Pass:\n");
   fprintf(fp, "    [-b beamwidth]      beam width (by state num)             (guessed)\n");
   fprintf(fp, "                        (0: full search, -1: force guess)\n");
 #ifdef WPAIR
@@ -224,7 +249,7 @@ j_output_argument_help(FILE *fp)
   fprintf(fp, "    [-1pass]            do 1st pass only, omit 2nd pass\n");
   fprintf(fp, "    [-inactive]         recognition process not active on startup\n");
 
-  fprintf(fp, "\n Search Parameters (Second Pass):\n");
+  fprintf(fp, "\n Search Parameters for the Second Pass:\n");
   fprintf(fp, "    [-b2 hyponum]       word envelope beam width (by hypo num) (%d)\n",jconf->search_root->pass2.enveloped_bestfirst_width);
   fprintf(fp, "    [-n N]              # of sentence to find                 (%d)\n", jconf->search_root->pass2.nbest);
   fprintf(fp, "    [-output N]         # of sentence to output               (%d)\n",jconf->search_root->output.output_hypo_maxnum);
@@ -244,6 +269,15 @@ j_output_argument_help(FILE *fp)
   fprintf(fp, "    [-iwsp]             insert sp for all word end (multipath)(off)\n");
   fprintf(fp, "    [-iwsppenalty]      trans. penalty for iwsp (multipath)   (%.1f)\n", jconf->am_root->iwsp_penalty);
 
+  fprintf(fp, "\n Short-pause Segmentation:\n");
+  fprintf(fp, "    [-spsegment]        enable short-pause segmentation\n");
+  fprintf(fp, "    [-spdur]            length threshold of sp frames         (%d)\n", jconf->search_root->successive.sp_frame_duration);
+#ifdef SPSEGMENT_NAIST
+  fprintf(fp, "    [-spmargin]         backstep margin on speech trigger     (%d)\n", jconf->search_root->successive.sp_margin);
+  fprintf(fp, "    [-spdelay]          delay on speech trigger               (%d)\n", jconf->search_root->successive.sp_delay);
+#endif
+  fprintf(fp, "    [-pausemodels str]  comma-delimited list of pause models for segment\n");
+
   fprintf(fp, "\n Graph Output with graph-oriented search:\n");
   fprintf(fp, "    [-lattice]          enable word graph (lattice) output\n");
   fprintf(fp, "    [-confnet]          enable confusion network output\n");
@@ -263,14 +297,6 @@ j_output_argument_help(FILE *fp)
   fprintf(fp, "    [-nographsearchdelay] disable it (default)\n");
 #endif
 
-  fprintf(fp, "\n On-the-fly Decoding: (default: on=mic/net off=files)\n");
-  fprintf(fp, "    [-realtime]         turn on, input streamed with MAP-CMN\n");
-  fprintf(fp, "    [-norealtime]       turn off, input buffered with sentence CMN\n");
-  fprintf(fp, "    [-cmnload file]     load initial CMN param from file on startup\n");
-  fprintf(fp, "    [-cmnsave file]     save CMN param to file after each input\n");
-  fprintf(fp, "    [-cmnnoupdate]      not update CMN param while recog. (use with -cmnload)\n");
-  fprintf(fp, "    [-cmnmapweight]     weight value of initial cm for MAP-CMN (%6.2f)\n", jconf->am_root->analysis.cmn_map_weight);
-
   fprintf(fp, "\n Forced Alignment:\n");
   fprintf(fp, "    [-walign]           optionally output word alignments\n");
   fprintf(fp, "    [-palign]           optionally output phoneme alignments\n");
@@ -287,18 +313,11 @@ j_output_argument_help(FILE *fp)
 #endif
 #endif /* CONFIDENCE_MEASURE */
   fprintf(fp, "\n Message Output:\n");
-  fprintf(fp, "    [-separatescore]    (n-gram) output LM and AM score independently\n");
-  fprintf(fp, "    [-quiet]            reduce output to only word string\n");
+  fprintf(fp, "    [-fallback1pass]    use 1st pass result when search failed\n");
   fprintf(fp, "    [-progout]          progressive output in 1st pass\n");
   fprintf(fp, "    [-proginterval]     interval of progout in msec           (%d)\n", jconf->search_root->output.progout_interval);
-  fprintf(fp, "    [-demo]             equal to \"-quiet -progout\"\n");
-  fprintf(fp, "\n Others:\n");
-  fprintf(fp, "    [-C jconffile]      load options from jconf file\n");
-  fprintf(fp, "    [-debug]            (for debug) dump numerous log\n");
-  fprintf(fp, "    [-check (wchmm|trellis)] (for debug) check internal structure\n");
-  fprintf(fp, "    [-check triphone]   triphone mapping check\n");
-  fprintf(fp, "    [-setting]          print engine configuration and exit\n");
-  fprintf(fp, "    [-help]             print this message and exit\n");
+
+  fprintf(fp, "\n-------------------------------------------------\n");
 
   j_jconf_free(jconf);
 
