@@ -1,7 +1,5 @@
 /**
  * @file   htk_hmm.h
- * @author Akinobu LEE
- * @date   Thu Feb 10 19:36:47 2005
  *
  * <EN>
  * @brief Data structures for handling HTK %HMM definition
@@ -15,13 +13,16 @@
  * Ƥޤ
  * </JA>
  *
- * $Revision: 1.6 $
+ * @author Akinobu LEE
+ * @date   Thu Feb 10 19:36:47 2005
+ *
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -41,7 +42,6 @@
 /// Delimiter string for parsing %HMM definition file
 #define HMMDEF_DELM " \t\n<>"
 
-
 /**
  * @defgroup hmminfo HTK HMM definition
  * <EN>
@@ -80,16 +80,16 @@
 #define SPMODEL_NAME_DEFAULT "sp"
 
 /// Length limit of HMM name (including ones generated in Julius)
-#define MAX_HMMNAME_LEN 128
+#define MAX_HMMNAME_LEN 64
 
 /// Specify method of calculating approximated acoustic score at inter-word context pseudo phones on word edge
 enum iwcd_type {
+  IWCD_UNDEF,			///< not specified explicitly
   IWCD_MAX,			///< Use maximum score among context variants
   IWCD_AVG,			///< Use average score among context variants
   IWCD_NBEST			///< Use average of N-best scores among context variants
 };
 
-
 /* options info */
 
 /// Stream information (although current Julius supports only single stream)
@@ -107,7 +107,6 @@ typedef struct {
   short param_type;		///< Type of parameter , see also htk_defs.h
 } HTK_HMM_Options;
 
-
 /// %HMM transition table
 typedef struct _HTK_HMM_trans {
   char *name;			///< Name (NULL if not defined as Macro)
@@ -178,7 +177,6 @@ typedef struct {
   /* GCODEBOOK *book;*/		/* pointer to the corresponding codebook in hmminfo */
 } GS_SET;
 
-
 /**
  * @defgroup cdset Context-Dependent HMM set
  * <EN>
@@ -249,7 +247,6 @@ typedef struct {
 } HMM_CDSET_INFO;
 //@}
 
-
 /**
  * @ingroup cdset
  *
@@ -355,14 +352,14 @@ typedef struct {
    * @name Misc. model information
    */
   //@{
+  boolean need_multipath; ///< TRUE if this model needs multipath handling
+  boolean multipath;		///< TRUE if this model is treated in multipath mode
   boolean is_triphone;		///< TRUE if this is triphone model
   boolean is_tied_mixture;	///< TRUE if this is tied-mixture model
   short cdset_method;		///< Selected method of computing pseudo phones in iwcd_type
   short cdmax_num;		///< Number of N-best states when IWCD_NBEST
   HMM_Logical *sp;		///< Link to short pause model
-#ifdef MULTIPATH_VERSION
-  LOGPROB iwsp_penalty;		///< Extra ransition penalty for interword skippable short pause insertion
-#endif
+  LOGPROB iwsp_penalty;		///< Extra ransition penalty for interword skippable short pause insertion for multi-path mode
   boolean variance_inversed;	///< TRUE if variances are inversed
   
   int totalmixnum;		///< Total number of defined mixtures
@@ -381,8 +378,6 @@ typedef struct {
   //@}
 } HTK_HMM_INFO;
 
-
-
 /* init_phmm.c */
 void htk_hmm_set_pause_model(HTK_HMM_INFO *hmminfo, char *spmodel_name);
 /* rdhmmdef.c */
@@ -424,16 +419,17 @@ void def_regtree_macro(char *name, FILE *fp, HTK_HMM_INFO *hmm);
 /* rdhmmdef_hmmlist.c */
 boolean rdhmmlist(FILE *fp, HTK_HMM_INFO *hmminfo);
 /* put_htkdata_info.c */
-void put_htk_trans(HTK_HMM_Trans *t);
-void put_htk_var(HTK_HMM_Var *v);
-void put_htk_dens(HTK_HMM_Dens *d);
-void put_htk_state(HTK_HMM_State *s);
-void put_htk_hmm(HTK_HMM_Data *h);
-void put_logical_hmm(HMM_Logical *l);
-void print_hmmdef_info(HTK_HMM_INFO *);
+void put_htk_trans(FILE *fp, HTK_HMM_Trans *t);
+void put_htk_var(FILE *fp, HTK_HMM_Var *v);
+void put_htk_dens(FILE *fp, HTK_HMM_Dens *d);
+void put_htk_state(FILE *fp, HTK_HMM_State *s);
+void put_htk_hmm(FILE *fp, HTK_HMM_Data *h);
+void put_logical_hmm(FILE *fp, HMM_Logical *l);
+void print_hmmdef_info(FILE *fp, HTK_HMM_INFO *);
+
 HTK_HMM_INFO *hmminfo_new();
 boolean hmminfo_free(HTK_HMM_INFO *);
-void init_hmminfo(HTK_HMM_INFO *hmminfo, char *filename, char *mapfile, Value *para);
+boolean init_hmminfo(HTK_HMM_INFO *hmminfo, char *filename, char *mapfile, Value *para);
 HTK_HMM_Data *htk_hmmdata_lookup_physical(HTK_HMM_INFO *, char *);
 HMM_Logical *htk_hmmdata_lookup_logical(HTK_HMM_INFO *, char *);
 void hmm_add_physical_to_logical(HTK_HMM_INFO *);
@@ -442,12 +438,11 @@ void hmm_add_pseudo_phones(HTK_HMM_INFO *hmminfo);
 void make_hmm_basephone_list(HTK_HMM_INFO *hmminfo);
 
 /* HMM type check functions */
+boolean htk_hmm_has_several_arc_on_edge(HTK_HMM_INFO *hmminfo);
 boolean check_hmm_limit(HTK_HMM_Data *dt);
 boolean check_all_hmm_limit(HTK_HMM_INFO *hmm);
 boolean check_hmm_options(HTK_HMM_INFO *hmm);
-#ifdef MULTIPATH_VERSION
 boolean is_skippable_model(HTK_HMM_Data *d);
-#endif
 
 /* CCD related */
 boolean guess_if_cd_hmm(HTK_HMM_INFO *hmm);
@@ -473,7 +468,7 @@ HTK_HMM_Trans *hmm_logical_trans(HMM_Logical *lg);
 #include <sent/htk_param.h>
 boolean check_param_coherence(HTK_HMM_INFO *hmm, HTK_Param *pinfo);
 boolean check_param_basetype(HTK_HMM_INFO *hmm, HTK_Param *pinfo);
-HTK_Param *new_param_check_and_adjust(HTK_HMM_INFO *hmm, HTK_Param *pinfo, boolean vflag);
+int param_check_and_adjust(HTK_HMM_INFO *hmm, HTK_Param *pinfo, boolean vflag);
 
 
 /* binary format */
