@@ -1,7 +1,5 @@
 /**
  * @file   adin_esd.c
- * @author Akinobu LEE
- * @date   Sun Feb 13 16:18:26 2005
  *
  * <JA>
  * @brief  ͥåȥϡEnlightened Sound Daemon (EsounD) β
@@ -20,12 +18,15 @@
  * to configure script.
  * </EN>
  *
+ * @author Akinobu LEE
+ * @date   Sun Feb 13 16:18:26 2005
+ *
  * $Revision: 1.2 $
  * 
  */
 /*
  * Copyright (c) 2004-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -56,10 +57,10 @@ adin_mic_standby(int sfreq, void *dummy)
   snprintf(name_buf, ESD_NAME_MAX, "julius%d", getpid());
 
   /* open stream */
-  j_printf("opening socket, format = 0x%08x at %d Hz id=%s\n", format, sfreq, name_buf);
+  jlog("adin_esd: opening socket, format = 0x%08x at %d Hz id=%s\n", format, sfreq, name_buf);
   sock = esd_record_stream_fallback(format, sfreq, NULL, name_buf);
   if (sock <= 0) {
-    j_printerr("adin_mic_standby: failed to connect to esd\n");
+    jlog("Error: adin_esd: failed to connect to esd\n");
     return FALSE;
   }
 
@@ -72,7 +73,7 @@ adin_mic_standby(int sfreq, void *dummy)
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-adin_mic_start()
+adin_mic_begin()
 {
   return(TRUE);
 }
@@ -83,7 +84,7 @@ adin_mic_start()
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-adin_mic_stop()
+adin_mic_end()
 {
   return TRUE;
 }
