@@ -1,7 +1,5 @@
 /**
  * @file   adin_mic_darwin_coreaudio.c
- * @author Masatomo Hashimoto
- * @date   Wed Oct 12 11:31:27 2005
  * 
  * <JA>
  * @brief  adin microphone library for CoreAudio API
@@ -28,7 +26,10 @@
  * 
  * </EN>
  * 
- * $Revision: 1.1.1.1 $
+ * @author Masatomo Hashimoto
+ * @date   Wed Oct 12 11:31:27 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 
@@ -43,7 +44,7 @@
  *
  */
 
-/* $Id: adin_mic_darwin_coreaudio.c,v 1.1.1.1 2005/11/17 11:11:49 sumomo Exp $ */
+/* $Id: adin_mic_darwin_coreaudio.c,v 1.2 2007/12/18 08:45:50 sumomo Exp $ */
 
 #include <CoreAudio/CoreAudio.h>
 #include <AudioUnit/AudioUnit.h>
@@ -87,11 +88,14 @@ static AudioBufferList* BufList;
 static AudioBufferList BufListBackup;
 static AudioBufferList* BufListConverted;
 
+#ifndef boolean
+typedef unsigned char boolean;
+#endif
 
 static void printStreamInfo(AudioStreamBasicDescription* desc) {
-  j_printf("----- details of stream -----\n");
-  j_printf("sample rate: %f\n", desc->mSampleRate);
-  j_printf("format flags: %s%s%s%s%s%s%s\n", 
+  jlog("Stat: adin_darwin: ----- details of stream -----\n");
+  jlog("Stat: adin_darwin: sample rate: %f\n", desc->mSampleRate);
+  jlog("Stat: adin_darwin: format flags: %s%s%s%s%s%s%s\n", 
 	   desc->mFormatFlags & kAudioFormatFlagIsFloat ? 
 	   "[float]" : "",
 	   desc->mFormatFlags & kAudioFormatFlagIsBigEndian ? 
@@ -107,12 +111,12 @@ static void printStreamInfo(AudioStreamBasicDescription* desc) {
 	   desc->mFormatFlags & kAudioFormatFlagsAreAllClear ? 
 	   "[all clear]" : ""
 	   );
-  j_printf("bytes per packet: %d\n", desc->mBytesPerPacket);
-  j_printf("frames per packet: %d\n", desc->mFramesPerPacket);
-  j_printf("bytes per frame: %d\n", desc->mBytesPerFrame);
-  j_printf("channels per frame: %d\n", desc->mChannelsPerFrame);
-  j_printf("bits per channel: %d\n", desc->mBitsPerChannel);
-  j_printf("-----------------------------------\n");
+  jlog("Stat: adin_darwin: bytes per packet: %d\n", desc->mBytesPerPacket);
+  jlog("Stat: adin_darwin: frames per packet: %d\n", desc->mFramesPerPacket);
+  jlog("Stat: adin_darwin: bytes per frame: %d\n", desc->mBytesPerFrame);
+  jlog("Stat: adin_darwin: channels per frame: %d\n", desc->mChannelsPerFrame);
+  jlog("Stat: adin_darwin: bits per channel: %d\n", desc->mBitsPerChannel);
+  jlog("Stat: adin_darwin: -----------------------------------\n");
 }
 
 static void printAudioBuffer(AudioBuffer* buf) {
@@ -120,7 +124,7 @@ static void printAudioBuffer(AudioBuffer* buf) {
   int i;
   Sample* p = (Sample*)(buf->mData);
   for (i = 0; i < sz; i++) {
-    j_printf("%d ", p[i]);
+    printf("%d ", p[i]);
   }
 }
 
@@ -130,15 +134,15 @@ allocateAudioBufferList(UInt32 data_bytes, UInt32 nsamples, UInt32 nchan) {
   AudioBufferList* bufl;
 
 #ifdef DEBUG
-  j_printf("allocateAudioBufferList: data_bytes:%d nsamples:%d nchan:%d\n",
+  jlog("Stat: adin_darwin: allocateAudioBufferList: data_bytes:%d nsamples:%d nchan:%d\n",
 	   data_bytes, nsamples, nchan);
 #endif
 
   bufl = (AudioBufferList*)(malloc(sizeof(AudioBufferList)));
 
   if(bufl == NULL) {
-    j_printerr("allocateAudioBufferList: failed\n");
-    exit(1);
+    jlog("Erorr: adin_darwin: allocateAudioBufferList: failed\n");
+    return NULL;
   }
 
   bufl->mNumberBuffers = nchan;
@@ -150,8 +154,8 @@ allocateAudioBufferList(UInt32 data_bytes, UInt32 nsamples, UInt32 nchan) {
     bufl->mBuffers[i].mData = malloc(data_bytes * nsamples);
     
     if(bufl->mBuffers[i].mData == NULL) {
-      j_printerr("allocateAudioBufferList: malloc for mBuffers[%d] failed\n", i);
-      exit(1);
+      jlog("Erorr: adin_darwin: allocateAudioBufferList: malloc for mBuffers[%d] failed\n", i);
+      return NULL;
     }
   }
   return bufl;
@@ -174,7 +178,7 @@ ConvInputProc(AudioConverterRef inConv,
   pthread_mutex_lock(&MutexInput);
 
 #ifdef DEBUG
-  j_printf("ConvInputProc: required %d packets\n", nPacketsRequired);
+  jlog("Stat: adin_darwin: ConvInputProc: required %d packets\n", nPacketsRequired);
 #endif
 
   while(NumSamplesAvailable == 0){
@@ -184,13 +188,13 @@ ConvInputProc(AudioConverterRef inConv,
   for(i = 0; i < BufList->mNumberBuffers; i++) {
     n = BufList->mBuffers[i].mDataByteSize;
     if (nBytesProvided != 0 && nBytesProvided != n) {
-      j_printerr("WARNING: buffer size mismatch\n");
+      jlog("Warning: adin_darwin: buffer size mismatch\n");
     }
     nBytesProvided = n;
   }
 
 #ifdef DEBUG
-  j_printf("ConvInputProc: %d bytes in buffer\n", nBytesProvided);
+  jlog("Stat: adin_darwin: ConvInputProc: %d bytes in buffer\n", nBytesProvided);
 #endif
 
   for(i = 0; i < BufList->mNumberBuffers; i++) {
@@ -217,14 +221,14 @@ ConvInputProc(AudioConverterRef inConv,
   *ioNumDataPackets = ioData->mBuffers[0].mDataByteSize / InputBytesPerPacket;
 
 #ifdef DEBUG
-  j_printf("ConvInputProc: provided %d packets\n", *ioNumDataPackets);
+  jlog("Stat: adin_darwin: ConvInputProc: provided %d packets\n", *ioNumDataPackets);
 #endif
 
   NumSamplesAvailable = 
     nBytesProvided / BytesPerSample - *ioNumDataPackets * InputSamplesPerPacket;
 
 #ifdef DEBUG
-  j_printf("ConvInputProc: %d samples available\n", NumSamplesAvailable);
+  jlog("Stat: adin_darwin: ConvInputProc: %d samples available\n", NumSamplesAvailable);
 #endif
 
   pthread_mutex_unlock(&MutexInput);
@@ -269,7 +273,7 @@ InputProc(void* inRefCon,
   pthread_cond_signal(&CondInput);
 
   /*
-  j_printf("InputProc: %d bytes filled (BufList)\n", 
+  jlog("Stat: adin_darwin: InputProc: %d bytes filled (BufList)\n", 
 	  BufList->mBuffers[0].mDataByteSize);
   */
 
@@ -278,14 +282,14 @@ InputProc(void* inRefCon,
 
 
 /* initialize default sound device */
-bool adin_mic_standby(int sfreq, void* dummy) {
+boolean adin_mic_standby(int sfreq, void* dummy) {
   OSStatus status;
   UInt32 propertySize;
   char deviceName[DEVICE_NAME_LEN];
   struct AudioStreamBasicDescription inDesc;
   int err;
 
-  j_printf("adin_mic_standby: sample rate = %d\n", sfreq);
+  jlog("Stat: adin_darwin: sample rate = %d\n", sfreq);
 
   if (CoreAudioInit) 
     return TRUE;
@@ -301,7 +305,7 @@ bool adin_mic_standby(int sfreq, void* dummy) {
   halout = FindNextComponent(NULL, &haloutDesc);
 
   if(halout == NULL) {
-    j_printerr("no HALOutput component found\n");
+    jlog("Error: adin_darwin: no HALOutput component found\n");
     return FALSE;
   }
 
@@ -317,7 +321,7 @@ bool adin_mic_standby(int sfreq, void* dummy) {
 				&enableIO,
 				sizeof(enableIO));
     if (status != noErr) {
-      j_printerr("cannot set InputUnit's EnableIO(Input)\n");
+      jlog("Error: adin_darwin: cannot set InputUnit's EnableIO(Input)\n");
       return FALSE;
     }
 
@@ -329,7 +333,7 @@ bool adin_mic_standby(int sfreq, void* dummy) {
 				&enableIO,
 				sizeof(enableIO));
     if (status != noErr) {
-      j_printerr("cannot set InputUnit's EnableIO(Output)\n");
+      jlog("Error: adin_darwin: cannot set InputUnit's EnableIO(Output)\n");
       return FALSE;
     }
 
@@ -340,12 +344,12 @@ bool adin_mic_standby(int sfreq, void* dummy) {
 	                            &propertySize,
 				    &InputDeviceID);
   if (status != noErr) {
-    j_printerr("cannot get default input device\n");
+    jlog("Error: adin_darwin: cannot get default input device\n");
     return FALSE;
   }
 
   if (InputDeviceID == kAudioDeviceUnknown) {
-    j_printerr("no available input device found\n");
+    jlog("Error: adin_darwin: no available input device found\n");
     return FALSE;
 
   } else {
@@ -361,7 +365,7 @@ bool adin_mic_standby(int sfreq, void* dummy) {
 				    &propertySize,
 				    deviceName);
     if (status != noErr) {
-      j_printerr("cannot get device name\n");
+      jlog("Error: adin_darwin: cannot get device name\n");
       return FALSE;
     }
 
@@ -373,7 +377,7 @@ bool adin_mic_standby(int sfreq, void* dummy) {
 				  sizeof(InputDeviceID));
 
     if (status != noErr) {
-      j_printerr("cannot bind default input device to AudioUnit\n");
+      jlog("Error: adin_darwin: cannot bind default input device to AudioUnit\n");
       return FALSE;
     }
 
@@ -386,7 +390,7 @@ bool adin_mic_standby(int sfreq, void* dummy) {
 				    &propertySize,
 				    &inDesc);
     if (status != noErr) {
-      j_printerr("cannot get input device's stream format\n");
+      jlog("Error: adin_darwin: cannot get input device's stream format\n");
       return FALSE;
     }
 
@@ -400,17 +404,17 @@ bool adin_mic_standby(int sfreq, void* dummy) {
 				    &propertySize,
 				    &bufferFrameSize);
     if (status != noErr) {
-      j_printerr("cannot get input device's buffer frame size\n");
+      jlog("Error: adin_darwin: cannot get input device's buffer frame size\n");
       return FALSE;
     }
 
-    j_printf("using device \"%s\" for input\n", deviceName);
-    j_printf("\tsample rate %f\n\t%ld channels\n\t%ld-bit sample\n",
+    jlog("Stat: adin_darwin: using device \"%s\" for input\n", deviceName);
+    jlog("Stat: adin_darwin: sample rate %f\n\t%ld channels\n\t%ld-bit sample\n",
 	    inDesc.mSampleRate,
 	    inDesc.mChannelsPerFrame,
 	    inDesc.mBitsPerChannel);
 
-    j_printf("\t%d buffer frames\n", bufferFrameSize);
+    jlog("Stat: adin_darwin: %d buffer frames\n", bufferFrameSize);
 
 
     printStreamInfo(&inDesc);
@@ -440,7 +444,7 @@ bool adin_mic_standby(int sfreq, void* dummy) {
 				  propertySize
 				  );
     if (status != noErr) {
-      j_printerr("cannot set InputUnit's stream format\n");
+      jlog("Error: adin_darwin: cannot set InputUnit's stream format\n");
       return FALSE;
     }
 
@@ -451,7 +455,7 @@ bool adin_mic_standby(int sfreq, void* dummy) {
     InputDeviceBufferSamples = 
       bufferFrameSize * InputSamplesPerPacket * InputFramesPerPacket;
 
-    j_printf("input device's buffer size (# of samples): %d\n", 
+    jlog("Stat: adin_darwin: input device's buffer size (# of samples): %d\n", 
 	     InputDeviceBufferSamples);
 
     AudioStreamBasicDescription outDesc;
@@ -476,8 +480,8 @@ bool adin_mic_standby(int sfreq, void* dummy) {
 
     status = AudioConverterNew(&inDesc, &outDesc, &Converter);
     if (status != noErr){
-      j_printerr("cannot create audio converter\n");
-      exit(1);
+      jlog("Error: adin_darwin: cannot create audio converter\n");
+      return FALSE;
     }
 
     /*
@@ -493,8 +497,8 @@ bool adin_mic_standby(int sfreq, void* dummy) {
 					 kAudioConverterChannelMap,
 					 sizeof(chmap), chmap);
       if (status != noErr){
-	j_printerr("cannot set audio converter's channel map\n");
-	exit(1);
+	jlog("cannot set audio converter's channel map\n");
+	return FALSE;
       }
     }
     */
@@ -504,16 +508,17 @@ bool adin_mic_standby(int sfreq, void* dummy) {
 				kAudioConverterSampleRateConverterQuality,
 				sizeof(ConvQuality), &ConvQuality);
     if (status != noErr){
-      j_printerr("cannot set audio converter quality\n");
-      exit(1);
+      jlog("Error: adin_darwin: cannot set audio converter quality\n");
+      return FALSE;
     }
 
 
-    //j_printf("audio converter generated\n");
+    //jlog("Stat: adin_darwin: audio converter generated\n");
 
     /* allocate buffers */
     BufList = allocateAudioBufferList(inDesc.mBitsPerChannel / BITS_PER_BYTE, 
 				      InputDeviceBufferSamples, 1);
+    if (BufList == NULL) return FALSE;
 
     BufListBackup.mNumberBuffers = BufList->mNumberBuffers;
 
@@ -523,16 +528,17 @@ bool adin_mic_standby(int sfreq, void* dummy) {
     BufListBackup.mBuffers[0].mData = BufList->mBuffers[0].mData;
 
     BufListConverted = allocateAudioBufferList(BytesPerSample, BUF_SAMPLES, 1);
-    //j_printf("buffers allocated\n");
+    if (BufListConverted == NULL) return FALSE;
+    //jlog("Stat: adin_darwin: buffers allocated\n");
 
     err = pthread_mutex_init(&MutexInput, NULL);
     if (err) {
-      j_printerr("cannot init mutex\n");
+      jlog("Error: adin_darwin: cannot init mutex\n");
       return FALSE;
     }
     err = pthread_cond_init(&CondInput, NULL);
     if (err) {
-      j_printerr("cannot init condition variable\n");
+      jlog("Error: adin_darwin: cannot init condition variable\n");
       return FALSE;
     }
 
@@ -549,27 +555,27 @@ bool adin_mic_standby(int sfreq, void* dummy) {
 
     status = AudioUnitInitialize(InputUnit);
     if (status != noErr){
-      j_printerr("InputUnit initialize failed\n");
-      exit(1);
+      jlog("Error: adin_darwin: InputUnit initialize failed\n");
+      return FALSE;
     }
 
   }
 
   CoreAudioInit = TRUE;
 
-  j_printf("CoreAudio: initialized\n");
+  jlog("Stat: adin_darwin: CoreAudio: initialized\n");
 
   return TRUE;
 }
 
-bool adin_mic_start(){ return TRUE; }
-bool adin_mic_stop(){ return TRUE; }
+boolean adin_mic_begin(){ return TRUE; }
+boolean adin_mic_end(){ return TRUE; }
 
 int adin_mic_read(void *buffer, int nsamples) {
   OSStatus status;
 
 #ifdef DEBUG
-  j_printf("adin_mic_read: %d samples required\n", nsamples);
+  jlog("Stat: adin_darwin: read: %d samples required\n", nsamples);
 #endif
 
   if (!CoreAudioHasInputDevice) 
@@ -590,7 +596,7 @@ int adin_mic_read(void *buffer, int nsamples) {
   numDataPacketsNeeded = npackets < capacity ? npackets : capacity;
 
 #ifdef DEBUG
-  j_printf("adin_mic_read: numDataPacketsNeeded=%d\n", numDataPacketsNeeded);
+  jlog("Stat: adin_darwin: numDataPacketsNeeded=%d\n", numDataPacketsNeeded);
 #endif
 
   status = AudioConverterFillComplexBuffer(Converter, 
@@ -601,12 +607,12 @@ int adin_mic_read(void *buffer, int nsamples) {
 					   NULL // packet description
 					   );
   if (status != noErr) {
-    j_printerr("AudioConverterFillComplexBuffer: failed\n");
+    jlog("Error: adin_darwin: AudioConverterFillComplexBuffer: failed\n");
     return -1;
   }
 
 #ifdef DEBUG
-  j_printf("adin_mic_read: %d bytes filled (BufListConverted)\n", 
+  jlog("Stat: adin_darwin: %d bytes filled (BufListConverted)\n", 
 	   BufListConverted->mBuffers[0].mDataByteSize);
 #endif
 
@@ -615,7 +621,7 @@ int adin_mic_read(void *buffer, int nsamples) {
   pthread_mutex_lock(&MutexInput);
 
 #ifdef DEBUG
-  j_printf("adin_mic_read: provided samples: %d\n", providedSamples);
+  jlog("Stat: adin_darwin: provided samples: %d\n", providedSamples);
 #endif
 
   Sample* dst_data = (Sample*)buffer;
@@ -629,13 +635,13 @@ int adin_mic_read(void *buffer, int nsamples) {
     if (dst_data[i] == 0) count++;
   }
 
-  //j_printf("%d zero samples\n", count);
+  //jlog("Stat: adin_darwin: %d zero samples\n", count);
 
 
   pthread_mutex_unlock(&MutexInput);
 
 #ifdef DEBUG
-  j_printf("adin_mic_read: EXIT: %d samples provided\n", providedSamples);
+  jlog("Stat: adindarwin: EXIT: %d samples provided\n", providedSamples);
 #endif
 
   return providedSamples;
