@@ -1,7 +1,5 @@
 /**
  * @file   adin_mic_freebsd.c
- * @author Akinobu LEE
- * @date   Sun Feb 13 16:18:26 2005
  *
  * <JA>
  * @brief  ޥ (FreeBSD)
@@ -38,13 +36,16 @@
  * Tested on FreeBSD 3.2-RELEASE with snd driver.
  * </EN>
  *
- * $Revision: 1.2 $
+ * @author Akinobu LEE
+ * @date   Sun Feb 13 16:18:26 2005
+ *
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -61,7 +62,11 @@
 #include <poll.h>
 
 /* sound header */
+#ifdef HAVE_SYS_SOUNDCARD_H
+#include <sys/soundcard.h>
+#elif HAVE_MACHINE_SOUNDCARD_H
 #include <machine/soundcard.h>
+#endif
 
 /// Default device name, can be overridden by AUDIODEV environment variable
 #define DEFAULT_DEVICE "/dev/dsp"
@@ -93,18 +98,21 @@ adin_mic_standby(int sfreq, void *arg)
   /* set device name */
   if ((devname = getenv("AUDIODEV")) == NULL) {
     devname = defaultdev;
+    jlog("Stat: adin_freebsd: device name = %s\n", devname);
+  } else {
+    jlog("Stat: adin_freebsd: device name obtained from AUDIODEV: %s\n", devname);
   }
   
   /* open device */
   if ((audio_fd = open(devname, O_RDONLY)) == -1) {
-    perror("adin_mic_standby: open");
+    jlog("Error: adin_freebsd: failed to open %s\n", devname);
     return(FALSE);
   }
 
   /* check whether soundcard can record 16bit data */
   /* and set fmt */
   if (ioctl(audio_fd, SNDCTL_DSP_GETFMTS, &fmt_can) == -1) {
-    perror("adin_mic_standby: sndctl_dsp_getfmts");
+    jlog("Error: adin_freebsd: failed to get formats from audio device\n");
     return(FALSE);
   }
 #ifdef WORDS_BIGENDIAN
@@ -122,14 +130,14 @@ adin_mic_standby(int sfreq, void *arg)
     fmt = fmt2;
     need_swap = TRUE;
   } else {
-    fprintf(stderr, "adin_mic_standby: 16bit recording not supported\n");
+    jlog("Error: adin_freebsd: 16bit recording not supported on this device\n");
     return FALSE;
   }
 #ifdef DEBUG
   if (need_swap) {
-    j_printf("samples need swap\n");
+    jlog("Stat: adin_freebsd: samples need swap\n");
   } else {
-    j_printf("samples need not swap\n");
+    jlog("Stat: adin_freebsd: samples need not swap\n");
   }
 #endif
   
@@ -138,41 +146,41 @@ adin_mic_standby(int sfreq, void *arg)
   /* re-open for recording */
   /* open device */
   if ((audio_fd = open(devname, O_RDONLY)) == -1) {
-    perror("adin_mic_standby: open");
+    jlog("Error: adin_freebsd: failed to open %s", devname);
     return(FALSE);
   }
   /* set format, samplerate, channels */
   rfmt = fmt;
   if (ioctl(audio_fd, SNDCTL_DSP_SETFMT, &rfmt) == -1) {
-    perror("adin_mic_standby: sndctl_dsp_setfmt");
+    jlog("Error: adin_freebsd: failed to get available formats from device\n");
     return(FALSE);
   }
   if (rfmt != fmt) {
-    fprintf(stderr, "adin_mic_standby: 16bit recording not supported\n");
+    jlog("Error: adin_freebsd: 16bit recording is not supported on this device\n");
     return FALSE;
   }
 
   stereo = 0;			/* mono */
   if (ioctl(audio_fd, SNDCTL_DSP_STEREO, &stereo) == -1) {
-    perror("adin_mic_standby: sndctl_dsp_stereo (mono)");
+    jlog("Error: adin_freebsd: failed to set monoral recording\n");
     return(FALSE);
   }
   if (stereo != 0) {
-    fprintf(stderr,"adin_mic_standby: monoral recording not supported\n");
+    jlog("Error: adin_freebsd: monoral recording not supported on this device\n");
     return FALSE;
   }
 
   samplerate = sfreq;
   if (ioctl(audio_fd, SNDCTL_DSP_SPEED, &samplerate) == -1) {
-    fprintf(stderr, "adin_mic_standby: sndctl_dsp_speed (%dHz)\n", sfreq);
+    jlog("Erorr: adin_freebsd: failed to set sample rate to %dHz\n", sfreq);
     return(FALSE);
   }
   if (samplerate < sfreq - FREQALLOWRANGE || samplerate > sfreq + FREQALLOWRANGE) {
-    fprintf(stderr,"adin_mic_standby: couldn't set sampling rate to near %dHz. (%d)\n", sfreq, samplerate);
+    jlog("Error: adin_freebsd: failed to set sampling rate to near %dHz. (%d)\n", sfreq, samplerate);
     return FALSE;
   }
   if (samplerate != sfreq) {
-    fprintf(stderr,"adin_mic_standby: set sampling rate to %dHz\n", samplerate);
+    jlog("Warning: adin_freebsd: specified sampling rate was %dHz but set to %dHz, \n", sfreq, samplerate);
   }
 
   /* set polling status */
@@ -188,13 +196,8 @@ adin_mic_standby(int sfreq, void *arg)
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-adin_mic_start()
+adin_mic_begin()
 {
-  char buf[2];
-
-  /* Read 1 sample (and ignore it) to tell the audio device start recording.
-     (If you knows better way, teach me...) */
-  read(audio_fd, buf, 2);
   return(TRUE);
 }
 
@@ -204,7 +207,7 @@ adin_mic_start()
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-adin_mic_stop()
+adin_mic_end()
 {
   /*
    * Not reset device on each end of speech, just let the buffer overrun...
@@ -237,7 +240,7 @@ adin_mic_read(SP16 *buf, int sampnum)
   poll(fds, 1, POLLINTERVAL);
   /* get actual sample num in the device buffer */
   if (ioctl(audio_fd, SNDCTL_DSP_GETISPACE, &info) == -1) {
-    perror("adin_mic_read: sndctl_dsp_getispace");
+    jlog("Error: adin_freebsd: adin_mic_read: sndctl_dsp_getispace");
     return(-2);
   }
   
@@ -246,7 +249,7 @@ adin_mic_read(SP16 *buf, int sampnum)
   if (size > info.bytes) size = info.bytes;
   cnt = read(audio_fd, buf, size);
   if ( cnt < 0 ) {
-    perror("adin_mic_read: read error\n");
+    jlog("Error: adin_freebsd: adin_mic_read: read error\n");
     return ( -2 );
   }
   cnt /= sizeof(short);
