@@ -44,7 +44,7 @@
  * @author Akinobu LEE
  * @date   Sun Feb 13 16:18:26 2005
  *
- * $Revision: 1.3 $
+ * $Revision: 1.5 $
  * 
  */
 /*
@@ -78,6 +78,72 @@ static int count;		///< Poll descriptor count
 #define MAXPOLLINTERVAL 300	///< Read timeout in msec.
 
 
+/** 
+ * Output detailed device information.
+ * 
+ * @param pcm_name [in] device name string
+ * @param handle [in] pcm audio handler
+ * 
+ */
+static void
+output_card_info(char *pcm_name, snd_pcm_t *handle)
+{
+  int err;
+  snd_ctl_t *ctl;
+  snd_ctl_card_info_t *info;
+  snd_pcm_info_t *pcminfo;
+  snd_ctl_card_info_alloca(&info);
+  snd_pcm_info_alloca(&pcminfo);
+  char ctlname[30];
+  int card;
+  
+  /* get PCM information to set current device and subdevice name */
+  if ((err = snd_pcm_info(handle, pcminfo)) < 0) {
+    jlog("Warning: adin_alsa: failed to obtain pcm info\n");
+    jlog("Warning: adin_alsa: skip output of detailed audio device info\n");
+    return;
+  }
+  /* open control associated with the pcm device name */
+  card = snd_pcm_info_get_card(pcminfo);
+  if (card < 0) {
+    strcpy(ctlname, "default");
+  } else {
+    snprintf(ctlname, 30, "hw:%d", card);
+  }
+  if ((err = snd_ctl_open(&ctl, ctlname, 0)) < 0) {
+    jlog("Warning: adin_alsa: failed to open control device \"%s\", \n", ctlname);
+    jlog("Warning: adin_alsa: skip output of detailed audio device info\n");
+    return;
+  }
+  /* get its card info */
+  if ((err = snd_ctl_card_info(ctl, info)) < 0) {
+    jlog("Warning: adin_alsa: unable to get card info for %s\n", ctlname);
+    jlog("Warning: adin_alsa: skip output of detailed audio device info\n");
+    snd_ctl_close(ctl);
+    return;
+  }
+
+  /* get detailed PCM information of current device from control */
+  if ((err = snd_ctl_pcm_info(ctl, pcminfo)) < 0) {
+    jlog("Error: adin_alsa: unable to get pcm info from card control\n");
+    jlog("Warning: adin_alsa: skip output of detailed audio device info\n");
+    snd_ctl_close(ctl);
+    return;
+  }
+  /* output */
+  jlog("Stat: \"%s\": %s [%s] device %s [%s] %s\n",
+       pcm_name,
+       snd_ctl_card_info_get_id(info),
+       snd_ctl_card_info_get_name(info),
+       snd_pcm_info_get_id(pcminfo),
+       snd_pcm_info_get_name(pcminfo),
+       snd_pcm_info_get_subdevice_name(pcminfo));
+
+  /* close controller */
+  snd_ctl_close(ctl);
+
+}
+
 /** 
  * Device initialization: check device capability and open for recording.
  * 
@@ -189,6 +255,7 @@ adin_mic_standby(int sfreq, void *dummy)
 #else
     unsigned int period_time, period_time_current;
     snd_pcm_uframes_t chunk_size;
+    boolean has_current_period;
 #endif
     boolean force = FALSE;
     
@@ -210,11 +277,13 @@ adin_mic_standby(int sfreq, void *dummy)
       return(FALSE);
     }
 #else    
+    has_current_period = TRUE;
     if ((err = snd_pcm_hw_params_get_period_time(hwparams, &period_time_current, &dir)) < 0) {
-      jlog("Error: adin_alsa: cannot get current period time\n");
-      return(FALSE);
+      has_current_period = FALSE;
+    }
+    if (has_current_period) {
+      jlog("Stat: adin_alsa: current latency time: %d msec\n", period_time_current / 1000);
     }
-    //jlog("Stat: adin_alsa: current latency time: %d msec\n", period_time_current / 1000);
 #endif
 
     /* set period time (near value will be used) */
@@ -238,9 +307,9 @@ adin_mic_standby(int sfreq, void *dummy)
     jlog("Stat: Audio I/O Latency = %d msec (data fragment = %d frames)\n", actual_size * 1000 / (actual_rate * sizeof(SP16)), actual_size / sizeof(SP16));
 #else
     period_time = latency * 1000;
-    if (!force && period_time > period_time_current) {
-      jlog("Stat: adin_alsa: current latency (%dms) is shorter than %dms, leave it\n", period_time_current / 1000, latency);
-      period_time = period_time_current;
+    if (!force && has_current_period && period_time > period_time_current) {
+	jlog("Stat: adin_alsa: current latency (%dms) is shorter than %dms, leave it\n", period_time_current / 1000, latency);
+	period_time = period_time_current;
     } else {
       if ((err = snd_pcm_hw_params_set_period_time_near(handle, hwparams, &period_time, 0)) < 0) {
 	jlog("Error: adin_alsa: cannot set PCM record period time to %d msec (%s)\n", period_time / 1000, snd_strerror(err));
@@ -286,43 +355,7 @@ adin_mic_standby(int sfreq, void *dummy)
 #endif
 
   /* output status */
-  {
-    snd_ctl_t *ctl;
-    snd_ctl_card_info_t *info;
-    snd_pcm_info_t *pcminfo;
-    snd_ctl_card_info_alloca(&info);
-    snd_pcm_info_alloca(&pcminfo);
-
-    /* open control associated with the pcm device name */
-    if ((err = snd_ctl_open(&ctl, pcm_name, 0)) < 0) {
-      jlog("Error: adin_alsa: unable to open control device for %s\n", pcm_name);
-      return(FALSE);
-    }
-    /* get its card info */
-    if ((err = snd_ctl_card_info(ctl, info)) < 0) {
-      jlog("Error: adin_alsa: unable to get card info for %s\n", pcm_name);
-      return(FALSE);
-    }
-    /* get PCM information to set current device and subdevice name */
-    if ((err = snd_pcm_info(handle, pcminfo)) < 0) {
-      jlog("Error: adin_alsa: unable to get pcm info\n");
-      return FALSE;
-    }
-    /* get detailed PCM information of current device from control */
-    if ((err = snd_ctl_pcm_info(ctl, pcminfo)) < 0) {
-      jlog("Error: adin_alsa: unable to get pcm info\n");
-      return FALSE;
-    }
-    /* output */
-    jlog("Stat: \"%s\": %s [%s] device %s [%s] %s\n",
-	 pcm_name,
-	 snd_ctl_card_info_get_id(info),
-	 snd_ctl_card_info_get_name(info),
-	 snd_pcm_info_get_id(pcminfo),
-	 snd_pcm_info_get_name(pcminfo),
-	 snd_pcm_info_get_subdevice_name(pcminfo));
-    snd_ctl_close(ctl);
-  }
+  output_card_info(pcm_name, handle);
 
   return(TRUE);
 }
