@@ -1,7 +1,5 @@
 /**
  * @file   rdparam.c
- * @author Akinobu LEE
- * @date   Tue Feb 15 00:16:44 2005
  *
  * <JA>
  * @brief  HTK ħѥ᡼եɤ߹ߡ¤Τοդ
@@ -24,13 +22,16 @@
  * these qualifiers are removed from its parameter type code.
  * </EN>
  *
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 00:16:44 2005
+ *
  * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -61,7 +62,7 @@ myread(char *buf, size_t unitbyte, int unitnum, FILE *fp)
 {
   size_t tmp;
   if ((tmp = myfread(buf, unitbyte, unitnum, fp)) < (size_t)unitnum) {
-    perror("Error: cannot read\n");
+    jlog("Error: rdparam: failed to read %d bytes\n", unitbyte * unitnum);
     return(FALSE);
   }
   /* swap if necessary */
@@ -104,9 +105,9 @@ read_param(FILE *fp, HTK_Param *pinfo)
   if(!myread((char *)&(hd->samplenum), sizeof(unsigned int), 1, fp)) return(FALSE);
   /* try to detect and read little-endian parameters from wav2mfcc... */
   if (hd->samplenum >= 60000) {	/* more than 10 minutes! */
-    j_printerr("Warning: data corrupted?: %d frames (more than 10 minutes)\n", hd->samplenum);
-    j_printerr("Warning: maybe MFCC made by wav2mfcc on a little-endian machine.\n");
-    j_printerr("retrying reading with endian conversion...\n");
+    jlog("Warning: rdparam: header says it has %d frames (more than 10 minutes)\n", hd->samplenum);
+    jlog("Warning: rdparam: it may be a little endian MFCC\n");
+    jlog("Warning: rdparam: now try reading with endian conversion\n");
     swap_bytes((char *)&(hd->samplenum), sizeof(unsigned int), 1);
     needswap = ! needswap;
   }
@@ -132,24 +133,15 @@ read_param(FILE *fp, HTK_Param *pinfo)
 
   buf = (char *)mymalloc(hd->sampsize);
 
+  /* allocate memory for vectors */
+  if (param_alloc(pinfo, pinfo->samplenum, pinfo->veclen) == FALSE) {
+    jlog("Error: rdparam: failed to allocate memory for reading MFCC\n");
+    return FALSE;
+  }
+
   /* read in parameter vector */
-  pinfo->parvec = (VECT **)mymalloc(sizeof(VECT *) * hd->samplenum);
-/* 
- *   {
- *     int size;
- *     float *ftmp;
- *     size = hd->samplenum * pinfo->veclen;
- *     if (size != 0) {
- *	 ftmp = (float *)mymalloc(sizeof(float) * size);
- *	 for(i=0;i<hd->samplenum;i++) {
- *	   pinfo->parvec[i] = &(ftmp[i * pinfo->veclen]);
- *	 }
- *     }
- *   }
- */
   /* needs conversion of integerized */
-  for (i=0;i<hd->samplenum;i++) {
-    pinfo->parvec[i] = (VECT *)mymalloc(sizeof(VECT) * pinfo->veclen);
+  for (i=0;i<pinfo->samplenum;i++) {
     if (hd->samptype & F_COMPRESS) {
       myread(buf, sizeof(short), hd->sampsize / sizeof(short), fp);
       p = buf;
@@ -176,51 +168,18 @@ read_param(FILE *fp, HTK_Param *pinfo)
     myread((char *)&c, sizeof(unsigned short), 1, fp);
   }
 
-  /*put_param(pinfo);*/
+  /*put_param(stdout, pinfo);*/
 
-  if (hd->samptype & F_COMPRESS) {
-    free(a);
-    free(b);
-  }
   free(buf);
+  if (hd->samptype & F_COMPRESS) {
+    free(b);
+    free(a);
+  }
 
   return(TRUE);
 
 }
 
-/** 
- * Allocate a new parameter.
- * 
- * @return pointer to the newly allocated area.
- */
-HTK_Param *
-new_param()
-{
-  HTK_Param *ret;
-  ret = (HTK_Param *)mymalloc(sizeof(HTK_Param));
-  ret->parvec = NULL;
-  ret->samplenum = 0;
-  return(ret);
-}
-
-/** 
- * Free the HTK parameter structure.
- * 
- * @param pinfo [in] parameter data to be destroyed.
- */
-void
-free_param(HTK_Param *pinfo)
-{
-  unsigned int i;
-  if (pinfo->samplenum > 0) {
-    for (i=0;i<pinfo->samplenum;i++) {
-      free(pinfo->parvec[i]);
-    }
-  }
-  if (pinfo->parvec != NULL) free(pinfo->parvec);
-  free(pinfo);
-}
-
 /** 
  * Top function to read a HTK parameter file.
  * 
