@@ -1,7 +1,5 @@
 /**
  * @file   mkcpair.c
- * @author Akinobu LEE
- * @date   Tue Feb 15 14:35:33 2005
  * 
  * <JA>
  * @brief  ʸˡ饫ƥФ
@@ -37,13 +35,16 @@
  * 
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 14:35:33 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -56,7 +57,7 @@
  * 
  * @param dinfo [i/o] DFA grammar, in which the category-pair matrix will be created.
  */
-void
+boolean
 extract_cpair(DFA_INFO *dinfo)
 {
   int i;
@@ -71,7 +72,8 @@ extract_cpair(DFA_INFO *dinfo)
     if ((dinfo->st[i].status & INITIAL_S) != 0) { /* arc from initial state */
       for (arc_r = dinfo->st[i].arc; arc_r; arc_r = arc_r->next) {
 	if (dinfo->is_sp[arc_r->label]) {
-	  j_error("Error: skippable sp should not appear at end of sentence\n");
+	  jlog("Error: mkcpair: skippable sp should not appear at end of sentence\n");
+	  return FALSE;
 	}
 	set_dfa_cp_end(dinfo, arc_r->label, TRUE);
       }
@@ -80,7 +82,8 @@ extract_cpair(DFA_INFO *dinfo)
       left = arc_l->label;
       if ((dinfo->st[arc_l->to_state].status & ACCEPT_S) != 0) {/* arc to accept state */
 	if (dinfo->is_sp[left]) {
-	  j_error("Error: skippable sp should not appear at beginning of sentence\n");
+	  jlog("Error: mkcpair: skippable sp should not appear at beginning of sentence\n");
+	  return FALSE;
 	}
 	set_dfa_cp_begin(dinfo, left, TRUE);
       }
@@ -91,7 +94,8 @@ extract_cpair(DFA_INFO *dinfo)
 	if (dinfo->is_sp[right]) {
 	  for (arc_r2 = dinfo->st[arc_r->to_state].arc; arc_r2; arc_r2 = arc_r2->next) {
 	    if (dinfo->is_sp[arc_r2->label]) { /* sp model continues twice */
-	      j_error("Error: skippable sp should not repeat\n");
+	      jlog("Error: mkcpair: skippable sp should not repeat\n");
+	      return FALSE;
 	    }
 	    set_dfa_cp(dinfo, arc_r2->label, left, TRUE);
 	  }
@@ -100,6 +104,8 @@ extract_cpair(DFA_INFO *dinfo)
 
     }
   }
+
+  return TRUE;
 }
 
 /** 
@@ -109,7 +115,7 @@ extract_cpair(DFA_INFO *dinfo)
  * @param src [in] DFA grammar to be appended to @a dst
  * @param coffset [in] category id in @a dst where the new data should be stored
  */
-void
+boolean
 cpair_append(DFA_INFO *dst, DFA_INFO *src, int coffset)
 {
   int i,j;
@@ -117,8 +123,9 @@ cpair_append(DFA_INFO *dst, DFA_INFO *src, int coffset)
   /* dst info must be already appended */
   /* [coffset..dst->term_num-1] is the new categories */
   if (dst->term_num - coffset != src->term_num) {
-    j_error("InternalError: append term num not match!: %d, %d, %d\n",
-	    dst->term_num, src->term_num, coffset);
+    jlog("Error: mkcpair: append term num not match!: %d, %d, %d\n",
+	       dst->term_num, src->term_num, coffset);
+    return FALSE;
   }
 
   /* allocate appended area */
@@ -132,4 +139,6 @@ cpair_append(DFA_INFO *dst, DFA_INFO *src, int coffset)
     set_dfa_cp_begin(dst, i, dfa_cp_begin(src, i-coffset));
     set_dfa_cp_end(dst, i, dfa_cp_end(src, i-coffset));
   }
+
+  return TRUE;
 }
