@@ -1,7 +1,5 @@
 /**
  * @file   init_phmm.c
- * @author Akinobu LEE
- * @date   Tue Feb 15 23:05:33 2005
  * 
  * <JA>
  * @brief  %HMM ե뤪HMMListޥåԥ󥰥եΥɤ߹ߤȽ
@@ -11,13 +9,16 @@
  * @brief  Load %HMM definition file and HMMList mapping file into memory and set it up.
  * </EN>
  * 
- * $Revision: 1.8 $
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 23:05:33 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -118,7 +119,7 @@ hmminfo_free(HTK_HMM_INFO *hmm)
  * @param namemapfile [in] file name of HMMList mapping file.
  * @param para [out] store acoustic analysis condition parameters if exist in hmmfilename.
  */
-void
+boolean
 init_hmminfo(HTK_HMM_INFO *hmminfo, char *hmmfilename, char *namemapfile, Value *para)
 {
   FILE *fp;
@@ -127,57 +128,65 @@ init_hmminfo(HTK_HMM_INFO *hmminfo, char *hmmfilename, char *namemapfile, Value
   ok_p = FALSE;
 
   /* read hmmdef file */
-  j_printerr("Reading in HMM definition...");
+  jlog("Stat: init_phmm: Reading in HMM definition\n");
   /* first, try ascii format */
   if ((fp = fopen_readfile(hmmfilename)) == NULL) {
-    j_error("failed to open %s\n",hmmfilename);
+    jlog("Error: init_phmm: failed to open %s\n",hmmfilename);
+    return FALSE;
   }
   if (rdhmmdef(fp, hmminfo) == TRUE) {
     ok_p = TRUE;
   }
   if (fclose_readfile(fp) < 0) {
-    j_error("failed to close %s\n", hmmfilename);
+    jlog("Error: init_phmm: failed to close %s\n", hmmfilename);
+    return FALSE;
   }
   if (ok_p == FALSE) {
     /* second, try binary format */
     if ((fp = fopen_readfile(hmmfilename)) == NULL) {
-      j_error("failed to open %s\n",hmmfilename);
+      jlog("Error: init_phmm: failed to open %s\n",hmmfilename);
+      return FALSE;
     }
     if (read_binhmm(fp, hmminfo, TRUE, para) == TRUE) {
       ok_p = TRUE;
     }
     if (fclose_readfile(fp) < 0) {
-      j_error("failed to close %s\n", hmmfilename);
+      jlog("Error: init_phmm: failed to close %s\n", hmmfilename);
+      return FALSE;
     }
   }
   if (ok_p == FALSE) {
-    j_error("failed to read %s\n", hmmfilename);
+    jlog("Error: init_phmm: failed to read %s\n", hmmfilename);
+    return FALSE;
   }
 
-  j_printerr("   defined HMMs: %5d\n", hmminfo->totalhmmnum);
+  jlog("Stat: init_phmm: defined HMMs: %5d\n", hmminfo->totalhmmnum);
 
   /* make mapping from logically named HMM to real defined HMM name */
   if (namemapfile != NULL) {
     /* use HMMList map file */
     if ((fp = fopen_readfile(namemapfile)) == NULL) {
-      j_error("failed to open %s\n",namemapfile);
+      jlog("Error: init_phmm: failed to open %s\n",namemapfile);
+      return FALSE;
     }
     if (rdhmmlist(fp, hmminfo) == FALSE) {
-      j_error("HMMList \"%s\" read error\n",namemapfile);
+      jlog("Error: init_phmm: HMMList \"%s\" read error\n",namemapfile);
+      return FALSE;
     }
     if (fclose_readfile(fp) < 0) {
-      j_error("failed to close %s\n", namemapfile);
+      jlog("Error: init_phmm: failed to close %s\n", namemapfile);
+      return FALSE;
     }
-    j_printerr("  logical names: %5d in HMMList\n", hmminfo->totallogicalnum);
+    jlog("Stat: init_phmm: logical names: %5d in HMMList\n", hmminfo->totallogicalnum);
   } else {
     /* add all names of physical HMMs as logical names */
     hmm_add_physical_to_logical(hmminfo);
-    j_printerr("  logical names: %5d\n", hmminfo->totallogicalnum);
+    jlog("Stat: init_phmm: logical names: %5d\n", hmminfo->totallogicalnum);
   }
 
   /* extract basephone */
   make_hmm_basephone_list(hmminfo);
-  j_printerr("    base phones: %5d used in logical\n", hmminfo->basephone.num);
+  jlog("Stat: init_phmm: base phones: %5d used in logical\n", hmminfo->basephone.num);
 
   /* Guess we need to handle context dependency */
   /* (word-internal CD is done statically, cross-word CD dynamically */
@@ -187,7 +196,9 @@ init_hmminfo(HTK_HMM_INFO *hmminfo, char *hmmfilename, char *namemapfile, Value
     hmminfo->is_triphone = FALSE;
   }
 
-  j_printerr("done\n");
+  jlog("Stat: init_phmm: finished reading HMM definitions\n");
+
+  return TRUE;
 }
 
 /** 
@@ -203,7 +214,7 @@ htk_hmm_set_pause_model(HTK_HMM_INFO *hmminfo, char *spmodel_name)
 
   l = htk_hmmdata_lookup_logical(hmminfo, spmodel_name);
   if (l == NULL) {
-    j_printerr("Warning: no model named as \"%s\"\n", spmodel_name);
+    jlog("Warning: init_phmm: no model named as \"%s\", no short pause model assigned\n", spmodel_name);
   }
   hmminfo->sp = l;
 }
