@@ -1,7 +1,5 @@
 /**
  * @file   rdhmmdef_dens.c
- * @author Akinobu LEE
- * @date   Wed Feb 16 01:43:43 2005
  * 
  * <JA>
  * @brief  HTK %HMM եɤ߹ߡʬ
@@ -11,17 +9,20 @@
  * @brief  Read HTK %HMM definition file: Gaussian density
  * </EN>
  * 
- * $Revision: 1.4 $
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 01:43:43 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* $Id: rdhmmdef_dens.c,v 1.4 2006/11/10 02:27:43 sumomo Exp $ */
+/* $Id: rdhmmdef_dens.c,v 1.2 2007/12/18 08:45:51 sumomo Exp $ */
 
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
@@ -90,8 +91,8 @@ dens_add(HTK_HMM_INFO *hmm, HTK_HMM_Dens *new)
       hmm->dn_root = aptree_make_root_node(new);
     } else {
       match = aptree_search_data(new->name, hmm->dn_root);
-      if (strmatch(match->name, new->name)) {
-	j_printerr("Error: ~m \"%s\" is already defined\n", new->name);
+      if (match != NULL && strmatch(match->name, new->name)) {
+	jlog("Error: rdhmmdef_dens: ~m \"%s\" is already defined\n", new->name);
 	rderr(NULL);
       } else {
 	aptree_add_entry(new->name, new, match->name, &(hmm->dn_root));
@@ -114,7 +115,7 @@ dens_lookup(HTK_HMM_INFO *hmm, char *keyname)
   HTK_HMM_Dens *d;
 
   d = aptree_search_data(keyname, hmm->dn_root);
-  if (strmatch(d->name, keyname)) {
+  if (d != NULL && strmatch(d->name, keyname)) {
     return d;
   } else {
     return NULL;
@@ -206,7 +207,7 @@ get_dens_data(FILE *fp, HTK_HMM_INFO *hmm)
     NoTokErr("missing macro name");
     tmp = dens_lookup(hmm, rdhmmdef_token);
     if (tmp == NULL) {
-      j_printerr("~m \"%s\" not defined\n", rdhmmdef_token);
+      jlog("Error: rdhmmdef_dens: ~m \"%s\" not defined\n", rdhmmdef_token);
       rderr(NULL);
     }
     read_token(fp);
