@@ -1,7 +1,5 @@
 /**
  * @file   rdhmmdef_state.c
- * @author Akinobu LEE
- * @date   Wed Feb 16 03:07:44 2005
  * 
  * <JA>
  * @brief  HTK %HMM եɤ߹ߡ
@@ -11,13 +9,16 @@
  * @brief  Read HTK %HMM definition file: state
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 03:07:44 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -69,8 +70,8 @@ state_add(HTK_HMM_INFO *hmm, HTK_HMM_State *new)
       hmm->st_root = aptree_make_root_node(new);
     } else {
       match = aptree_search_data(new->name, hmm->st_root);
-      if (strmatch(match->name, new->name)) {
-	j_printerr("Error: ~s \"%s\" is already defined\n", new->name);
+      if (match != NULL && strmatch(match->name, new->name)) {
+	jlog("Error: rdhmmdef_state: ~s \"%s\" is already defined\n", new->name);
 	rderr(NULL);
       } else {
 	aptree_add_entry(new->name, new, match->name, &(hmm->st_root));
@@ -94,7 +95,7 @@ state_lookup(HTK_HMM_INFO *hmm, char *keyname)
   HTK_HMM_State *s;
 
   s = aptree_search_data(keyname, hmm->st_root);
-  if (strmatch(s->name, keyname)) {
+  if (s != NULL && strmatch(s->name, keyname)) {
     return s;
   } else {
     return NULL;
@@ -168,18 +169,6 @@ state_read(FILE *fp, HTK_HMM_INFO *hmm)
     }
   }
 
-#if 0
-  /* diabled */
-  for (i=0;i<new->mix_num;i++) {
-    /* make error if no mixture was associated */
-    if (new->b[i] == NULL) {
-      j_printerr("Warning: lack of mixture component found, use previous insted...\n");
-      new->b[i] = new->b[i-1];
-    }
-  }
-#endif
-  
-  
   return (new);
 }
 
@@ -213,7 +202,7 @@ get_state_data(FILE *fp, HTK_HMM_INFO *hmm)
     NoTokErr("missing state macro name");
     tmp = state_lookup(hmm, rdhmmdef_token);
     if (tmp == NULL) {
-      j_printerr("Error: ~s \"%s\" not defined\n", rdhmmdef_token);
+      jlog("Error: rdhmmdef_state: ~s \"%s\" not defined\n", rdhmmdef_token);
       rderr(NULL);
     }
     read_token(fp);
