@@ -1,7 +1,5 @@
 /**
  * @file   rdhmmdef_tiedmix.c
- * @author Akinobu LEE
- * @date   Wed Feb 16 03:25:11 2005
  * 
  * <JA>
  * @brief  HTK %HMM եɤ߹ߡtied-mixtureǥκʬۥɥ֥å
@@ -11,13 +9,16 @@
  * @brief  Read HTK %HMM definition file: mixture codebook in tied-mixture model
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 03:25:11 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -41,7 +42,7 @@ codebook_lookup(HTK_HMM_INFO *hmm, char *keyname)
 
   if (hmm->codebook_root == NULL) return(NULL);
   book = aptree_search_data(keyname, hmm->codebook_root);
-  if (strmatch(book->name, keyname)) {
+  if (book != NULL && strmatch(book->name, keyname)) {
     return book;
   } else {
     return NULL;
@@ -62,8 +63,8 @@ codebook_add(HTK_HMM_INFO *hmm, GCODEBOOK *new)
     hmm->codebook_root = aptree_make_root_node(new);
   } else {
     match = aptree_search_data(new->name, hmm->codebook_root);
-    if (strmatch(match->name, new->name)) {
-      j_printerr("Error: ~s \"%s\" is already defined\n", new->name);
+    if (match != NULL && strmatch(match->name, new->name)) {
+      jlog("Error: rdhmmdef_tiedmix: ~s \"%s\" is already defined\n", new->name);
       rderr(NULL);
     } else {
       aptree_add_entry(new->name, new, match->name, &(hmm->codebook_root));
@@ -101,7 +102,7 @@ tmix_create_codebook_index(HTK_HMM_INFO *hmminfo, GCODEBOOK *book)
     sprintf(mixname, "%s%d", book->name, i + 1);
     if ((dtmp = dens_lookup(hmminfo, mixname)) == NULL) {
 /* 
- *	 j_printerr("Error: mixture \"%s\" (%dth mixture in codebook \"%s\") not found\n", mixname, i + 1, book->name);
+ *	 jlog("Error: mixture \"%s\" (%dth mixture in codebook \"%s\") not found\n", mixname, i + 1, book->name);
  *	 rderr(NULL);
  */
       book->d[i] = NULL;
@@ -111,8 +112,8 @@ tmix_create_codebook_index(HTK_HMM_INFO *hmminfo, GCODEBOOK *book)
     }
   }
   if (realbooknum < book->num) {
-    j_printerr("Warning: book [%s]: defined=%d < %d\n",
-	       book->name, realbooknum, book->num);
+    jlog("Warning: rdhmmdef_tiedmix: book [%s]: defined=%d < %d\n",
+	 book->name, realbooknum, book->num);
   }
   
   free(mixname);
