@@ -1,7 +1,5 @@
 /**
  * @file   ngram_lookup.c
- * @author Akinobu LEE
- * @date   Wed Feb 16 16:42:38 2005
  * 
  * <JA>
  * @brief  N-gramθåȥθ
@@ -11,13 +9,16 @@
  * @brief  Look up N-gram entries from its name string
  * </EN>
  * 
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 16:42:38 2005
+ *
  * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -65,7 +66,7 @@ ngram_lookup_word(NGRAM_INFO *ndata, char *wordstr)
 {
   int data;
   data = ptree_search_data(wordstr, ndata->root);
-  if (strcmp(wordstr, ndata->wname[data]) != 0) {
+  if (data == -1 || strcmp(wordstr, ndata->wname[data]) != 0) {
     return WORD_INVALID;
   } else {
     return(data);
@@ -87,7 +88,7 @@ make_ngram_ref(NGRAM_INFO *ndata, char *wstr)
 
   nw = ngram_lookup_word(ndata, wstr);
   if (nw == WORD_INVALID) {	/* not found */
-    j_printf("word %s not exist in N-gram, treat as <UNK>\n", wstr);
+    jlog("Warning: ngram_lookup: word %s not exist in N-gram vocabulary, treat as <UNK>\n", wstr);
     return(ndata->unk_id);
   } else {
     return(nw);
