@@ -1,7 +1,5 @@
 /**
  * @file   ngram_malloc.c
- * @author Akinobu LEE
- * @date   Wed Feb 16 16:48:56 2005
  * 
  * <JA>
  * @brief  N-gram¤ΤνݤȲ
@@ -11,13 +9,16 @@
  * @brief  Initial memory allocation and free for N-gram stucture
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 16:48:56 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -35,10 +36,28 @@ ngram_info_new()
   NGRAM_INFO *new;
 
   new = (NGRAM_INFO *)mymalloc(sizeof(NGRAM_INFO));
+  new->bo_wt_1 = NULL;
+  new->p_2 = NULL;
 
   return(new);
 }
 
+static void
+free_ngram_tuple(NGRAM_TUPLE_INFO *t)
+{
+  if (t->is24bit) {
+    if (t->bgn_upper) free(t->bgn_upper);
+    if (t->bgn_lower) free(t->bgn_lower);
+  } else {
+    if (t->bgn) free(t->bgn);
+  }
+  if (t->num) free(t->num);
+  if (t->nnid2wid) free(t->nnid2wid);
+  free(t->prob);
+  if (t->bo_wt) free(t->bo_wt);
+  if (t->nnid2ctid_upper) free(t->nnid2ctid_upper);
+  if (t->nnid2ctid_lower) free(t->nnid2ctid_lower);
+}
 /** 
  * Free N-gram data.
  * 
@@ -47,6 +66,7 @@ ngram_info_new()
 void
 ngram_info_free(NGRAM_INFO *ndata)
 {
+  int i;
   /* bin test only */
   /* free word names */
   if (ndata->from_bin) {
@@ -59,37 +79,15 @@ ngram_info_free(NGRAM_INFO *ndata)
     }
     free(ndata->wname);
   }
-  /* free 1-gram */
-  free(ndata->p);
-  free(ndata->bo_wt_lr);
-  free(ndata->bo_wt_rl);
-  free(ndata->n2_bgn);
-  free(ndata->n2_num);
-  /* free 2-gram */
-  free(ndata->n2tonid);
-  free(ndata->p_lr);
-  free(ndata->p_rl);
-  switch(ndata->version) {
-  case 3:
-    free(ndata->bo_wt_rrl);
-    free(ndata->n3_bgn);
-    free(ndata->n3_num);
-    break;
-  case 4:
-    free(ndata->n2bo_upper);
-    free(ndata->n2bo_lower);
-    free(ndata->bo_wt_rrl);
-    free(ndata->n3_bgn_upper);
-    free(ndata->n3_bgn_lower);
-    free(ndata->n3_num);
-    break;
+  /* free 2-gram for the 1st pass */
+  if (ndata->bo_wt_1) free(ndata->bo_wt_1);
+  if (ndata->p_2) free(ndata->p_2);
+  /* free n-gram */
+  for(i=0;i<ndata->n;i++) {
+    free_ngram_tuple(&(ndata->d[i]));
   }
-  /* free 3-gram */
-  free(ndata->n3tonid);
-  free(ndata->p_rrl);
-  /* free index tree */
+  /* free name index tree */
   free_ptree(ndata->root);
   /* free whole */
   free(ndata);
 }
-
