@@ -1,7 +1,5 @@
 /**
  * @file   gprune_beam.c
- * @author Akinobu LEE
- * @date   Thu Feb 17 03:27:53 2005
  * 
  * <JA>
  * @brief  祬ʬ۷׻: Gaussian pruning (beam algorithm)
@@ -39,13 +37,16 @@
  * calc_tied_mix() or calc_mix().
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 03:27:53 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -53,8 +54,9 @@
 #include <sent/htk_hmm.h>
 #include <sent/htk_param.h>
 #include <sent/hmm.h>
-#include <sent/gprune.h>
-#include "globalvars.h"
+#include <sent/hmm_calc.h>
+
+#define TEST2
 
 /*
 
@@ -113,33 +115,31 @@
      
 */
 
-/* pruning threshold */
-static LOGPROB *dimthres;	///< Threshold for each dimension (inversed)
-static int dimthres_num;	///< Length of above
-
-static boolean *mixcalced;	///< Mark which Gaussian has been computed
-
 /** 
  * Clear per-dimension thresholds.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  */
 static void
-clear_dimthres()
+clear_dimthres(HMMWork *wrk)
 {
   int i;
-  for(i=0;i<dimthres_num;i++) dimthres[i] = 0.0;
+  for(i=0;i<wrk->dimthres_num;i++) wrk->dimthres[i] = 0.0;
 }
 
 /** 
  * Set beam thresholds by adding TMBEAMWIDTH to the maximum values
  * of each dimension.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  */
 static void
-set_dimthres()
+set_dimthres(HMMWork *wrk)
 {
   int i;
-  for(i=0;i<dimthres_num;i++) dimthres[i] += TMBEAMWIDTH;
+  for(i=0;i<wrk->dimthres_num;i++) wrk->dimthres[i] += TMBEAMWIDTH;
 }
 
 /**
@@ -150,32 +150,36 @@ set_dimthres()
  * for future pruning.  The pruning itself is not performed here.
  * This function will be used to compute the first N Gaussians.
  * 
+ * @param wrk [i/o] HMM computation work area
  * @param binfo [in] Gaussian density
  * 
  * @return the output log probability.
  */
 static LOGPROB
-compute_g_beam_updating(HTK_HMM_Dens *binfo)
+compute_g_beam_updating(HMMWork *wrk, HTK_HMM_Dens *binfo)
 {
   VECT tmp, x;
   VECT *mean;
   VECT *var;
-  VECT *th = dimthres;
-  VECT *vec = OP_vec;
-  short veclen = OP_veclen;
+  VECT *th = wrk->dimthres;
+  VECT *vec = wrk->OP_vec;
+  short veclen = wrk->OP_veclen;
 
+#ifndef TEST2
   if (binfo == NULL) return(LOG_ZERO);
+#endif
+
   mean = binfo->mean;
   var = binfo->var->vec;
 
   tmp = 0.0;
   for (; veclen > 0; veclen--) {
     x = *(vec++) - *(mean++);
-    tmp += x * x / *(var++);
+    tmp += x * x * *(var++);
     if ( *th < tmp) *th = tmp;
     th++;
   }
-  return((tmp + binfo->gconst) / -2.0);
+  return((tmp + binfo->gconst) * -0.5);
 }
 
 /** 
@@ -186,53 +190,58 @@ compute_g_beam_updating(HTK_HMM_Dens *binfo)
  * that has been set by compute_g_beam_updating() and
  * set_dimthres().
  * 
+ * @param wrk [i/o] HMM computation work area
  * @param binfo [in] Gaussian density
  * 
  * @return the output log probability.
  */
 static LOGPROB
-compute_g_beam_pruning(HTK_HMM_Dens *binfo)
+compute_g_beam_pruning(HMMWork *wrk, HTK_HMM_Dens *binfo)
 {
   VECT tmp, x;
   VECT *mean;
   VECT *var;
-  VECT *th = dimthres;
-  VECT *vec = OP_vec;
-  short veclen = OP_veclen;
+  VECT *th = wrk->dimthres;
+  VECT *vec = wrk->OP_vec;
+  short veclen = wrk->OP_veclen;
 
+#ifndef TEST2
   if (binfo == NULL) return(LOG_ZERO);
+#endif
   mean = binfo->mean;
   var = binfo->var->vec;
 
   tmp = 0.0;
   for (; veclen > 0; veclen--) {
     x = *(vec++) - *(mean++);
-    tmp += x * x / *(var++);
+    tmp += x * x * *(var++);
     if ( tmp > *(th++)) {
       return LOG_ZERO;
     }
   }
-  return((tmp + binfo->gconst) / -2.0);
+  return((tmp + binfo->gconst) * -0.5);
 }
 
 
 /** 
  * Initialize and setup work area for Gaussian pruning by beam algorithm.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-gprune_beam_init()
+gprune_beam_init(HMMWork *wrk)
 {
   int i;
   /* maximum Gaussian set size = maximum mixture size */
-  OP_calced_maxnum = OP_hmminfo->maxmixturenum;
-  OP_calced_score = (LOGPROB *)mymalloc(sizeof(LOGPROB) * OP_gprune_num);
-  OP_calced_id = (int *)mymalloc(sizeof(int) * OP_gprune_num);
-  mixcalced = (boolean *)mymalloc(sizeof(int) * OP_calced_maxnum);
-  for(i=0;i<OP_calced_maxnum;i++) mixcalced[i] = FALSE;
-  dimthres_num = OP_hmminfo->opt.vec_size;
-  dimthres = (LOGPROB *)mymalloc(sizeof(LOGPROB) * dimthres_num);
+  wrk->OP_calced_maxnum = wrk->OP_hmminfo->maxmixturenum;
+  wrk->OP_calced_score = (LOGPROB *)mymalloc(sizeof(LOGPROB) * wrk->OP_gprune_num);
+  wrk->OP_calced_id = (int *)mymalloc(sizeof(int) * wrk->OP_gprune_num);
+  wrk->mixcalced = (boolean *)mymalloc(sizeof(int) * wrk->OP_calced_maxnum);
+  for(i=0;i<wrk->OP_calced_maxnum;i++) wrk->mixcalced[i] = FALSE;
+  wrk->dimthres_num = wrk->OP_hmminfo->opt.vec_size;
+  wrk->dimthres = (LOGPROB *)mymalloc(sizeof(LOGPROB) * wrk->dimthres_num);
 
   return TRUE;
 }
@@ -240,14 +249,16 @@ gprune_beam_init()
 /**
  * Free gprune_beam related work area.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  */
 void
-gprune_beam_free()
+gprune_beam_free(HMMWork *wrk)
 {
-  free(OP_calced_score);
-  free(OP_calced_id);
-  free(mixcalced);
-  free(dimthres);
+  free(wrk->OP_calced_score);
+  free(wrk->OP_calced_id);
+  free(wrk->mixcalced);
+  free(wrk->dimthres);
 }
 
 /** 
@@ -268,57 +279,76 @@ gprune_beam_free()
  * 
  * This can be called from calc_tied_mix() or calc_mix().
  * 
+ * @param wrk [i/o] HMM computation work area
  * @param g [in] set of Gaussian densities to compute the output probability
  * @param gnum [in] length of above
  * @param last_id [in] ID list of N-best mixture in previous input frame,
  * or NULL if not exist
  */
 void
-gprune_beam(HTK_HMM_Dens **g, int gnum, int *last_id)
+gprune_beam(HMMWork *wrk, HTK_HMM_Dens **g, int gnum, int *last_id)
 {
   int i, j, num = 0;
   LOGPROB score, thres;
 
   if (last_id != NULL) {	/* compute them first to form thresholds */
     /* 1. clear dimthres */
-    clear_dimthres();
+    clear_dimthres(wrk);
     /* 2. calculate first $OP_gprune_num and set initial thresholds */
-    for (j=0; j<OP_gprune_num; j++) {
+    for (j=0; j<wrk->OP_gprune_num; j++) {
       i = last_id[j];
-      score = compute_g_beam_updating(g[i]);
-      num = cache_push(i, score, num);
-      mixcalced[i] = TRUE;      /* mark them as calculated */
+#ifdef TEST2
+      if (!g[i]) {
+	score = LOG_ZERO;
+      } else {
+	score = compute_g_beam_updating(wrk, g[i]);
+      }
+      num = cache_push(wrk, i, score, num);
+#else
+      score = compute_g_beam_updating(wrk, g[i]);
+      num = cache_push(wrk, i, score, num);
+#endif
+      wrk->mixcalced[i] = TRUE;      /* mark them as calculated */
     }
     /* 3. set pruning thresholds for each dimension */
-    set_dimthres();
+    set_dimthres(wrk);
 
     /* 4. calculate the rest with pruning*/
     for (i = 0; i < gnum; i++) {
       /* skip calced ones in 1. */
-      if (mixcalced[i]) {
-        mixcalced[i] = FALSE;
+      if (wrk->mixcalced[i]) {
+        wrk->mixcalced[i] = FALSE;
         continue;
       }
+#ifdef TEST2
       /* compute with safe pruning */
-      score = compute_g_beam_pruning(g[i]);
+      if (!g[i]) continue;
+      score = compute_g_beam_pruning(wrk, g[i]);
       if (score > LOG_ZERO) {
-	num = cache_push(i, score, num);
+	num = cache_push(wrk, i, score, num);
       }
+#else
+      /* compute with safe pruning */
+      score = compute_g_beam_pruning(wrk, g[i]);
+      if (score > LOG_ZERO) {
+	num = cache_push(wrk, i, score, num);
+      }
+#endif
     }
   } else {			/* in case the last_id not available */
     /* at the first 0 frame */
     /* calculate with safe pruning */
     thres = LOG_ZERO;
     for (i = 0; i < gnum; i++) {
-      if (num < OP_gprune_num) {
-	score = compute_g_base(g[i]);
+      if (num < wrk->OP_gprune_num) {
+	score = compute_g_base(wrk, g[i]);
       } else {
-	score = compute_g_safe(g[i], thres);
+	score = compute_g_safe(wrk, g[i], thres);
 	if (score <= thres) continue;
       }
-      num = cache_push(i, score, num);
-      thres = OP_calced_score[num-1];
+      num = cache_push(wrk, i, score, num);
+      thres = wrk->OP_calced_score[num-1];
     }
   }
-  OP_calced_num = num;
+  wrk->OP_calced_num = num;
 }
