@@ -1,7 +1,5 @@
 /**
  * @file   gprune_heu.c
- * @author Akinobu LEE
- * @date   Thu Feb 17 05:44:52 2005
  * 
  * <JA>
  * @brief  祬ʬ۷׻: Gaussian pruning (heuristic algorithm)
@@ -37,13 +35,16 @@
  * calc_tied_mix() or calc_mix().
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 05:44:52 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -51,8 +52,7 @@
 #include <sent/htk_hmm.h>
 #include <sent/htk_param.h>
 #include <sent/hmm.h>
-#include <sent/gprune.h>
-#include "globalvars.h"
+#include <sent/hmm_calc.h>
 
 /* mail routines to calculate mixtures for given frame, and store to cache */
 /*
@@ -102,27 +102,24 @@
      
 */
 
-/* pruning threshold */
-static LOGPROB *backmax;	///< Backward sum of max for each dimension (inversed)
-static int backmax_num;		///< Length of above
-
-static boolean *mixcalced;	///< Mark which Gaussian has been computed
-
 /** 
  * Clear backmax information.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  */
 static void
-init_backmax()
+init_backmax(HMMWork *wrk)
 {
   int i;
-  for(i=0;i<backmax_num;i++) backmax[i] = 0;
+  for(i=0;i<wrk->backmax_num;i++) wrk->backmax[i] = 0;
 }
 
 /** 
  * Build backmax information for each frame, by summing up
  * current maximum values of each dimensions.
  * 
+ * @param wrk [i/o] HMM computation work area
  */
 /*                        |
  *  0 1 2 ... max-3 max-2 | max-1
@@ -133,13 +130,14 @@ init_backmax()
  *  .....     x+y     y     0.0
  */
 static void
-make_backmax()
+make_backmax(HMMWork *wrk)
 {
   int i;
-  backmax[backmax_num-1] = 0.0;
+
+  wrk->backmax[wrk->backmax_num-1] = 0.0;
   /* backmax[len-1] = backmax[len-1];*/
-  for(i=backmax_num-2;i>=0;i--) {
-    backmax[i] += backmax[i+1];
+  for(i=wrk->backmax_num-2;i>=0;i--) {
+    wrk->backmax[i] += wrk->backmax[i+1];
   }
   /*  for(i=0;i<=len;i++) {
     printf("backmax[%d]=%f\n",i,backmax[i]);
@@ -154,19 +152,20 @@ make_backmax()
  * for future pruning.  The pruning itself is not performed here.
  * This function will be used to compute the first N Gaussians.
  * 
+ * @param wrk [i/o] HMM computation work area
  * @param binfo [in] Gaussian density
  * 
  * @return the output log probability.
  */
 static LOGPROB
-compute_g_heu_updating(HTK_HMM_Dens *binfo)
+compute_g_heu_updating(HMMWork *wrk, HTK_HMM_Dens *binfo)
 {
   VECT tmp, x, sum = 0.0;
   VECT *mean;
   VECT *var;
-  VECT *bm = backmax;
-  VECT *vec = OP_vec;
-  short veclen = OP_veclen;
+  VECT *bm = wrk->backmax;
+  VECT *vec = wrk->OP_vec;
+  short veclen = wrk->OP_veclen;
 
   if (binfo == NULL) return(LOG_ZERO);
   mean = binfo->mean;
@@ -175,12 +174,12 @@ compute_g_heu_updating(HTK_HMM_Dens *binfo)
   tmp = 0.0;
   for (; veclen > 0; veclen--) {
     x = *(vec++) - *(mean++);
-    tmp = x * x / *(var++);
+    tmp = x * x * *(var++);
     sum += tmp;
     if ( *bm < tmp) *bm = tmp;
     bm++;
   }
-  return((sum + binfo->gconst) / -2.0);
+  return((sum + binfo->gconst) * -0.5);
 }
 
 /** 
@@ -191,20 +190,21 @@ compute_g_heu_updating(HTK_HMM_Dens *binfo)
  * that has been set by compute_g_heu_updating() and
  * make_backmax().
  * 
+ * @param wrk [i/o] HMM computation work area
  * @param binfo [in] Gaussian density
  * @param thres [in] threshold 
  * 
  * @return the output log probability.
  */
 static LOGPROB
-compute_g_heu_pruning(HTK_HMM_Dens *binfo, LOGPROB thres)
+compute_g_heu_pruning(HMMWork *wrk, HTK_HMM_Dens *binfo, LOGPROB thres)
 {
   VECT tmp, x;
   VECT *mean;
   VECT *var;
-  VECT *bm = backmax;
-  VECT *vec = OP_vec;
-  short veclen = OP_veclen;
+  VECT *bm = wrk->backmax;
+  VECT *vec = wrk->OP_vec;
+  short veclen = wrk->OP_veclen;
   LOGPROB fthres;
 
   if (binfo == NULL) return(LOG_ZERO);
@@ -216,33 +216,35 @@ compute_g_heu_pruning(HTK_HMM_Dens *binfo, LOGPROB thres)
   bm++;
   for (; veclen > 0; veclen--) {
     x = *(vec++) - *(mean++);
-    tmp += x * x / *(var++);
+    tmp += x * x * *(var++);
     if ( tmp + *bm > fthres) {
       return LOG_ZERO;
     }
     bm++;
   }
-  return((tmp + binfo->gconst) / -2.0);
+  return((tmp + binfo->gconst) * -0.5);
 }
 
 
 /** 
  * Initialize and setup work area for Gaussian pruning by heuristic algorithm.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-gprune_heu_init()
+gprune_heu_init(HMMWork *wrk)
 {
   int i;
   /* maximum Gaussian set size = maximum mixture size */
-  OP_calced_maxnum = OP_hmminfo->maxmixturenum;
-  OP_calced_score = (LOGPROB *)mymalloc(sizeof(LOGPROB) * OP_gprune_num);
-  OP_calced_id = (int *)mymalloc(sizeof(int) * OP_gprune_num);
-  mixcalced = (boolean *)mymalloc(sizeof(int) * OP_calced_maxnum);
-  for(i=0;i<OP_calced_maxnum;i++) mixcalced[i] = FALSE;
-  backmax_num = OP_hmminfo->opt.vec_size + 1;
-  backmax = (LOGPROB *)mymalloc(sizeof(LOGPROB) * backmax_num);
+  wrk->OP_calced_maxnum = wrk->OP_hmminfo->maxmixturenum;
+  wrk->OP_calced_score = (LOGPROB *)mymalloc(sizeof(LOGPROB) * wrk->OP_gprune_num);
+  wrk->OP_calced_id = (int *)mymalloc(sizeof(int) * wrk->OP_gprune_num);
+  wrk->mixcalced = (boolean *)mymalloc(sizeof(int) * wrk->OP_calced_maxnum);
+  for(i=0;i<wrk->OP_calced_maxnum;i++) wrk->mixcalced[i] = FALSE;
+  wrk->backmax_num = wrk->OP_hmminfo->opt.vec_size + 1;
+  wrk->backmax = (LOGPROB *)mymalloc(sizeof(LOGPROB) * wrk->backmax_num);
 
   return TRUE;
 }
@@ -250,14 +252,16 @@ gprune_heu_init()
 /**
  * Free gprune_heu related work area.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  */
 void
-gprune_heu_free()
+gprune_heu_free(HMMWork *wrk)
 {
-  free(OP_calced_score);
-  free(OP_calced_id);
-  free(mixcalced);
-  free(backmax);
+  free(wrk->OP_calced_score);
+  free(wrk->OP_calced_id);
+  free(wrk->mixcalced);
+  free(wrk->backmax);
 }
 
 /** 
@@ -279,42 +283,43 @@ gprune_heu_free()
  * 
  * This can be called from calc_tied_mix() or calc_mix().
  * 
+ * @param wrk [i/o] HMM computation work area
  * @param g [in] set of Gaussian densities to compute the output probability
  * @param gnum [in] length of above
  * @param last_id [in] ID list of N-best mixture in previous input frame,
  * or NULL if not exist
  */
 void
-gprune_heu(HTK_HMM_Dens **g, int gnum, int *last_id)
+gprune_heu(HMMWork *wrk, HTK_HMM_Dens **g, int gnum, int *last_id)
 {
   int i, j, num = 0;
   LOGPROB score, thres;
 
   if (last_id != NULL) {	/* compute them first to form thresholds */
     /* 1. clear backmax */
-    init_backmax();
+    init_backmax(wrk);
     /* 2. calculate first $OP_gprune_num with setting max for each dimension */
-    for (j=0; j<OP_gprune_num; j++) {
+    for (j=0; j<wrk->OP_gprune_num; j++) {
       i = last_id[j];
-      score = compute_g_heu_updating(g[i]);
-      num = cache_push(i, score, num);
-      mixcalced[i] = TRUE;      /* mark them as calculated */
+      score = compute_g_heu_updating(wrk, g[i]);
+      num = cache_push(wrk, i, score, num);
+      wrk->mixcalced[i] = TRUE;      /* mark them as calculated */
     }
     /* 3. set backmax for each dimension */
-    make_backmax();
+    make_backmax(wrk);
     /* 4. calculate the rest with pruning*/
-    thres = OP_calced_score[num-1];
+    thres = wrk->OP_calced_score[num-1];
     for (i = 0; i < gnum; i++) {
       /* skip calced ones in 1. */
-      if (mixcalced[i]) {
-        mixcalced[i] = FALSE;
+      if (wrk->mixcalced[i]) {
+        wrk->mixcalced[i] = FALSE;
         continue;
       }
       /* compute with safe pruning */
-      score = compute_g_heu_pruning(g[i], thres);
+      score = compute_g_heu_pruning(wrk, g[i], thres);
       if (score > LOG_ZERO) {
-	num = cache_push(i, score, num);
-	thres = OP_calced_score[num-1];
+	num = cache_push(wrk, i, score, num);
+	thres = wrk->OP_calced_score[num-1];
       }
     }
   } else {			/* in case the last_id not available */
@@ -322,15 +327,15 @@ gprune_heu(HTK_HMM_Dens **g, int gnum, int *last_id)
     /* calculate with safe pruning */
     thres = LOG_ZERO;
     for (i = 0; i < gnum; i++) {
-      if (num < OP_gprune_num) {
-	score = compute_g_base(g[i]);
+      if (num < wrk->OP_gprune_num) {
+	score = compute_g_base(wrk, g[i]);
       } else {
-	score = compute_g_safe(g[i], thres);
+	score = compute_g_safe(wrk, g[i], thres);
 	if (score <= thres) continue;
       }
-      num = cache_push(i, score, num);
-      thres = OP_calced_score[num-1];
+      num = cache_push(wrk, i, score, num);
+      thres = wrk->OP_calced_score[num-1];
     }
   }
-  OP_calced_num = num;
+  wrk->OP_calced_num = num;
 }
