@@ -1,7 +1,5 @@
 /**
  * @file   outprob.c
- * @author Akinobu LEE
- * @date   Fri Feb 18 18:45:21 2005
  * 
  * <JA>
  * @brief  ٷ׻μ¹ԤӾ֥٥륭å
@@ -45,13 +43,16 @@
  * in addition to this state-level cache.  See calc_tied_mix.c for details.
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Fri Feb 18 18:45:21 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -60,51 +61,46 @@
 #include <sent/htk_hmm.h>
 #include <sent/htk_param.h>
 #include <sent/hmm.h>
-#include <sent/gprune.h>
-#include "globalvars.h"
+#include <sent/hmm_calc.h>
 
 
-/* cache */
-static int statenum;		///< Local work area that holds total number of HMM states in the %HMM definition data
-static LOGPROB **outprob_cache = NULL; ///< State-level cache [t][stateid]
-static int allocframenum;	///< Allocated frames of the cache
-static int allocblock;		///< Allocation block size per allocateion of the cache
-static BMALLOC_BASE *croot;
-static LOGPROB *last_cache;	///< Local work are to hold cache list of current time
+
 #define LOG_UNDEF (LOG_ZERO - 1) ///< Value to be used as the initial cache value
 
 /** 
  * Initialize the cache data, should be called once on startup.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-outprob_cache_init()
+outprob_cache_init(HMMWork *wrk)
 {
-  statenum = OP_hmminfo->totalstatenum;
-  outprob_cache = NULL;
-  allocframenum = 0;
-  allocblock = OUTPROB_CACHE_PERIOD;
-  OP_time = -1;
-  croot = NULL;
+  wrk->statenum = wrk->OP_hmminfo->totalstatenum;
+  wrk->outprob_cache = NULL;
+  wrk->outprob_allocframenum = 0;
+  wrk->OP_time = -1;
+  wrk->croot = NULL;
   return TRUE;
 }
 
 /** 
  * Prepare cache for the next input, by clearing the existing cache.
  * 
+ * @param wrk [i/o] HMM computation work area
  * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-outprob_cache_prepare()
+outprob_cache_prepare(HMMWork *wrk)
 {
   int s,t;
 
   /* clear already allocated area */
-  for (t = 0; t < allocframenum; t++) {
-    for (s = 0; s < statenum; s++) {
-      outprob_cache[t][s] = LOG_UNDEF;
+  for (t = 0; t < wrk->outprob_allocframenum; t++) {
+    for (s = 0; s < wrk->statenum; s++) {
+      wrk->outprob_cache[t][s] = LOG_UNDEF;
     }
   }
   
@@ -114,10 +110,11 @@ outprob_cache_prepare()
 /** 
  * Expand the cache to time axis if needed.
  * 
+ * @param wrk [i/o] HMM computation work area
  * @param reqframe [in] required frame length
  */
 static void
-outprob_cache_extend(int reqframe)
+outprob_cache_extend(HMMWork *wrk, int reqframe)
 {
   int newnum;
   int size;
@@ -125,41 +122,43 @@ outprob_cache_extend(int reqframe)
   LOGPROB *tmpp;
 
   /* if enough length are already allocated, return immediately */
-  if (reqframe < allocframenum) return;
+  if (reqframe < wrk->outprob_allocframenum) return;
 
   /* allocate per certain period */
   newnum = reqframe + 1;
-  if (newnum < allocframenum + allocblock) newnum = allocframenum + allocblock;
-  size = (newnum - allocframenum) * statenum;
+  if (newnum < wrk->outprob_allocframenum + OUTPROB_CACHE_PERIOD) newnum = wrk->outprob_allocframenum + OUTPROB_CACHE_PERIOD;
+  size = (newnum - wrk->outprob_allocframenum) * wrk->statenum;
   
   /* allocate */
-  if (outprob_cache == NULL) {
-    outprob_cache = (LOGPROB **)mymalloc(sizeof(LOGPROB *) * newnum);
+  if (wrk->outprob_cache == NULL) {
+    wrk->outprob_cache = (LOGPROB **)mymalloc(sizeof(LOGPROB *) * newnum);
   } else {
-    outprob_cache = (LOGPROB **)myrealloc(outprob_cache, sizeof(LOGPROB *) * newnum);
+    wrk->outprob_cache = (LOGPROB **)myrealloc(wrk->outprob_cache, sizeof(LOGPROB *) * newnum);
   }
-  tmpp = (LOGPROB *)mybmalloc2(sizeof(LOGPROB) * size, &croot);
+  tmpp = (LOGPROB *)mybmalloc2(sizeof(LOGPROB) * size, &(wrk->croot));
   /* clear the new part */
-  for(t = allocframenum; t < newnum; t++) {
-    outprob_cache[t] = &(tmpp[(t - allocframenum) * statenum]);
-    for (s = 0; s < statenum; s++) {
-      outprob_cache[t][s] = LOG_UNDEF;
+  for(t = wrk->outprob_allocframenum; t < newnum; t++) {
+    wrk->outprob_cache[t] = &(tmpp[(t - wrk->outprob_allocframenum) * wrk->statenum]);
+    for (s = 0; s < wrk->statenum; s++) {
+      wrk->outprob_cache[t][s] = LOG_UNDEF;
     }
   }
 
-  /*j_printf("outprob cache: %d->%d\n", allocframenum, newnum);*/
-  allocframenum = newnum;
+  /*jlog("outprob cache: %d->%d\n", outprob_allocframenum, newnum);*/
+  wrk->outprob_allocframenum = newnum;
 }
 
 /**
  * Free work area for cache.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  */
 void
-outprob_cache_free()
+outprob_cache_free(HMMWork *wrk)
 {
-  if (croot != NULL) mybfree2(&croot);
-  if (outprob_cache != NULL) free(outprob_cache);
+  if (wrk->croot != NULL) mybfree2(&(wrk->croot));
+  if (wrk->outprob_cache != NULL) free(wrk->outprob_cache);
 }
 
 
@@ -173,66 +172,72 @@ outprob_cache_free()
  * (If you use GMS, the entity will be gms_state() instead.)
  *
  * The state-level cache is also consulted here.
+ *
+ * @param wrk [i/o] HMM computation work area
+ * @param t [in] time frame
+ * @param stateinfo [in] state information to compute the output probability
+ * @param param [in] input parameter vectors
  * 
  * @return output log probability.
  */
 LOGPROB
-outprob_state(
-     int t,			/* time frame */
-     HTK_HMM_State *stateinfo,	/* state info */
-     HTK_Param *param)		/* parameter */
+outprob_state(HMMWork *wrk, int t, HTK_HMM_State *stateinfo, HTK_Param *param)
 {
   LOGPROB outp;
+  int sid;
+
+  sid = stateinfo->id;
   
   /* set global values for outprob functions to access them */
-  OP_state = stateinfo;
-  OP_state_id = stateinfo->id;
-  OP_param = param;
-  if (OP_time != t) {
-    OP_last_time = OP_time;
-    OP_time = t;
-    OP_vec = param->parvec[t];
-    OP_veclen = param->veclen;
+  wrk->OP_state = stateinfo;
+  wrk->OP_state_id = sid;
+  wrk->OP_param = param;
+  if (wrk->OP_time != t) {
+    wrk->OP_last_time = wrk->OP_time;
+    wrk->OP_time = t;
+    wrk->OP_vec = param->parvec[t];
+    wrk->OP_veclen = param->veclen;
 
-    outprob_cache_extend(t);	/* extend cache if needed */
-    last_cache = outprob_cache[t]; /* reduce 2-d array access */
+    outprob_cache_extend(wrk, t);	/* extend cache if needed */
+    wrk->last_cache = wrk->outprob_cache[t]; /* reduce 2-d array access */
   }
   
   /* consult cache */
-  if ((outp = last_cache[OP_state_id]) == LOG_UNDEF) {
-    outp = last_cache[OP_state_id] = calc_outprob_state();
+  if ((outp = wrk->last_cache[sid]) == LOG_UNDEF) {
+    outp = wrk->last_cache[sid] = (*(wrk->calc_outprob_state))(wrk);
   }
   return(outp);
 }
 
-static LOGPROB *maxprobs;	///< Work area that holds N-best state scores for pseudo state set
-static int maxn;		///< Allocated length of above
-
 /** 
  * Initialize work area for outprob_cd_nbest().
  * 
+ * @param wrk [i/o] HMM computation work area
  * @param num [in] number of top states to be calculated.
  */
 void
-outprob_cd_nbest_init(int num)
+outprob_cd_nbest_init(HMMWork *wrk, int num)
 {
-  maxprobs = (LOGPROB *)mymalloc(sizeof(LOGPROB) * num);
-  maxn = num;
+  wrk->cd_nbest_maxprobs = (LOGPROB *)mymalloc(sizeof(LOGPROB) * num);
+  wrk->cd_nbest_maxn = num;
 }
 
 /**
  * Free work area for outprob_cd_nbest().
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  */
 void
-outprob_cd_nbest_free()
+outprob_cd_nbest_free(HMMWork *wrk)
 {
-  free(maxprobs);
+  free(wrk->cd_nbest_maxprobs);
 }
   
 /** 
  * Return average of N-beat outprob for pseudo state set.
  * 
+ * @param wrk [i/o] HMM computation work area
  * @param t [in] input frame
  * @param lset [in] pseudo state set
  * @param param [in] input parameter data
@@ -240,36 +245,36 @@ outprob_cd_nbest_free()
  * @return outprob log probability, average of top N states in @a lset.
  */
 static LOGPROB
-outprob_cd_nbest(int t, CD_State_Set *lset, HTK_Param *param)
+outprob_cd_nbest(HMMWork *wrk, int t, CD_State_Set *lset, HTK_Param *param)
 {
   LOGPROB prob;
   int i, k, n;
 
   n = 0;
   for(i=0;i<lset->num;i++) {
-    prob = outprob_state(t, lset->s[i], param);
-    /*printf("\t\t%d:%f\n", i, prob);*/
+    prob = outprob_state(wrk, t, lset->s[i], param);
+    /*jlog("\t\t%d:%f\n", i, prob);*/
     if (prob <= LOG_ZERO) continue;
-    if (n == 0 || prob <= maxprobs[n-1]) {
-      if (n == maxn) continue;
-      maxprobs[n] = prob;
+    if (n == 0 || prob <= wrk->cd_nbest_maxprobs[n-1]) {
+      if (n == wrk->cd_nbest_maxn) continue;
+      wrk->cd_nbest_maxprobs[n] = prob;
       n++;
     } else {
       for(k=0; k<n; k++) {
-	if (prob > maxprobs[k]) {
-	  memmove(&(maxprobs[k+1]), &(maxprobs[k]),
-		  sizeof(LOGPROB) * (n - k - ( (n == maxn) ? 1 : 0)));
-	  maxprobs[k] = prob;
+	if (prob > wrk->cd_nbest_maxprobs[k]) {
+	  memmove(&(wrk->cd_nbest_maxprobs[k+1]), &(wrk->cd_nbest_maxprobs[k]),
+		  sizeof(LOGPROB) * (n - k - ( (n == wrk->cd_nbest_maxn) ? 1 : 0)));
+	  wrk->cd_nbest_maxprobs[k] = prob;
 	  break;
 	}
       }
-      if (n < maxn) n++;
+      if (n < wrk->cd_nbest_maxn) n++;
     }
   }
   prob = 0.0;
   for(i=0;i<n;i++) {
-    /*printf("\t\t\t- %d: %f\n", i, maxprobs[i]);*/
-    prob += maxprobs[i];
+    /*jlog("\t\t\t- %d: %f\n", i, wrk->cd_nbest_maxprobs[i]);*/
+    prob += wrk->cd_nbest_maxprobs[i];
   }
   return(prob/(float)n);
 }
@@ -277,6 +282,7 @@ outprob_cd_nbest(int t, CD_State_Set *lset, HTK_Param *param)
 /** 
  * Return maximum outprob of the pseudo state set.
  * 
+ * @param wrk [i/o] HMM computation work area
  * @param t [in] input frame
  * @param lset [in] pseudo state set
  * @param param [in] input parameter data
@@ -284,13 +290,14 @@ outprob_cd_nbest(int t, CD_State_Set *lset, HTK_Param *param)
  * @return maximum output log probability among states in @a lset.
  */
 static LOGPROB
-outprob_cd_max(int t, CD_State_Set *lset, HTK_Param *param)
+outprob_cd_max(HMMWork *wrk, int t, CD_State_Set *lset, HTK_Param *param)
 {
   LOGPROB maxprob, prob;
   int i;
+
   maxprob = LOG_ZERO;
   for(i=0;i<lset->num;i++) {
-    prob = outprob_state(t, lset->s[i], param);
+    prob = outprob_state(wrk, t, lset->s[i], param);
     if (maxprob < prob) maxprob = prob;
   }
   return(maxprob);
@@ -299,6 +306,7 @@ outprob_cd_max(int t, CD_State_Set *lset, HTK_Param *param)
 /** 
  * Return average outprob of the pseudo state set.
  * 
+ * @param wrk [i/o] HMM computation work area
  * @param t [in] input frame
  * @param lset [in] pseudo state set
  * @param param [in] input parameter data
@@ -306,14 +314,14 @@ outprob_cd_max(int t, CD_State_Set *lset, HTK_Param *param)
  * @return average output log probability of states in @a lset.
  */
 static LOGPROB
-outprob_cd_avg(int t, CD_State_Set *lset, HTK_Param *param)
+outprob_cd_avg(HMMWork *wrk, int t, CD_State_Set *lset, HTK_Param *param)
 {
   LOGPROB sum, p;
   int i,j;
   sum = 0.0;
   j = 0;
   for(i=0;i<lset->num;i++) {
-    p = outprob_state(t, lset->s[i], param);
+    p = outprob_state(wrk, t, lset->s[i], param);
     if (p > LOG_ZERO) {
       sum += p;
       j++;
@@ -325,6 +333,7 @@ outprob_cd_avg(int t, CD_State_Set *lset, HTK_Param *param)
 /** 
  * Compute the log output probability of a pseudo state set.
  * 
+ * @param wrk [i/o] HMM computation work area
  * @param t [in] input frame
  * @param lset [in] pseudo state set
  * @param param [in] input parameter data
@@ -332,24 +341,20 @@ outprob_cd_avg(int t, CD_State_Set *lset, HTK_Param *param)
  * @return the computed log output probability.
  */
 LOGPROB
-outprob_cd(int t, CD_State_Set *lset, HTK_Param *param)
+outprob_cd(HMMWork *wrk, int t, CD_State_Set *lset, HTK_Param *param)
 {
   LOGPROB ret;
 
   /* select computation method */
-  switch(OP_hmminfo->cdset_method) {
+  switch(wrk->OP_hmminfo->cdset_method) {
   case IWCD_AVG:
-    ret = outprob_cd_avg(t, lset, param);
+    ret = outprob_cd_avg(wrk, t, lset, param);
     break;
   case IWCD_MAX:
-    ret = outprob_cd_max(t, lset, param);
+    ret = outprob_cd_max(wrk, t, lset, param);
     break;
   case IWCD_NBEST:
-    ret = outprob_cd_nbest(t, lset, param);
-    break;
-  default:
-    j_error("unknown cdhmm method!\n");
-    ret = 0;
+    ret = outprob_cd_nbest(wrk, t, lset, param);
     break;
   }
   return(ret);
@@ -358,7 +363,8 @@ outprob_cd(int t, CD_State_Set *lset, HTK_Param *param)
 
 /** 
  * Top function to compute the output probability of a HMM state.
- * 
+ *
+ * @param wrk [i/o] HMM computation work area
  * @param t [in] input frame
  * @param hmmstate [in] HMM state
  * @param param [in] input parameter data
@@ -366,11 +372,11 @@ outprob_cd(int t, CD_State_Set *lset, HTK_Param *param)
  * @return the computed log output probability.
  */
 LOGPROB
-outprob(int t, HMM_STATE *hmmstate, HTK_Param *param)
+outprob(HMMWork *wrk, int t, HMM_STATE *hmmstate, HTK_Param *param)
 {
   if (hmmstate->is_pseudo_state) {
-    return(outprob_cd(t, hmmstate->out.cdset, param));
+    return(outprob_cd(wrk, t, hmmstate->out.cdset, param));
   } else {
-    return(outprob_state(t, hmmstate->out.state, param));
+    return(outprob_state(wrk, t, hmmstate->out.state, param));
   }
 }
