@@ -1,7 +1,5 @@
 /**
  * @file   outprob_init.c
- * @author Akinobu LEE
- * @date   Thu Feb 17 13:35:37 2005
  * 
  * <JA>
  * @brief  ٷ׻롼νȥåȥå
@@ -33,13 +31,16 @@
  *      -# use outprob(t, hmmstate, param) to get output probability of a state
  * </EN>
  * 
- * $Revision: 1.5 $
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 13:35:37 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -47,19 +48,24 @@
 #include <sent/htk_hmm.h>
 #include <sent/htk_param.h>
 #include <sent/hmm.h>
-#include <sent/gprune.h>
+#include <sent/hmm_calc.h>
 
-#define GLOBAL_VARIABLE_DEFINE	///< Global variables is actually defined here
-#include "globalvars.h"
 
-
 /** 
  * Initialize and setup acoustic computation functions.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * @param hmminfo [in] HMM definition
+ * @param gshmm [in] GMS HMM definition if exist, or NULL if not
+ * @param gms_num [in] number of GMS HMM to compute (valid if gshmm != NULL)
+ * @param gprune_method [in] gaussian pruning method
+ * @param gprune_mixnum [in] number of pdf to compute at a codebook
+ * in gaussian pruning
+ * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-outprob_init(HTK_HMM_INFO *hmminfo,
+outprob_init(HMMWork *wrk, HTK_HMM_INFO *hmminfo,
 	     HTK_HMM_INFO *gshmm, int gms_num,
 	     int gprune_method, int gprune_mixnum
 	     )
@@ -84,64 +90,65 @@ outprob_init(HTK_HMM_INFO *hmminfo,
      and set the pointer to global */
   switch(gprune_method) {
   case GPRUNE_SEL_NONE:
-    compute_gaussset = gprune_none;
-    compute_gaussset_init = gprune_none_init;
-    compute_gaussset_free = gprune_none_free;
+    wrk->compute_gaussset = gprune_none;
+    wrk->compute_gaussset_init = gprune_none_init;
+    wrk->compute_gaussset_free = gprune_none_free;
     break;
   case GPRUNE_SEL_SAFE:
-    compute_gaussset = gprune_safe;
-    compute_gaussset_init = gprune_safe_init;
-    compute_gaussset_free = gprune_safe_free;
+    wrk->compute_gaussset = gprune_safe;
+    wrk->compute_gaussset_init = gprune_safe_init;
+    wrk->compute_gaussset_free = gprune_safe_free;
     break;
   case GPRUNE_SEL_HEURISTIC:
-    compute_gaussset = gprune_heu;
-    compute_gaussset_init = gprune_heu_init;
-    compute_gaussset_free = gprune_heu_free;
+    wrk->compute_gaussset = gprune_heu;
+    wrk->compute_gaussset_init = gprune_heu_init;
+    wrk->compute_gaussset_free = gprune_heu_free;
     break;
   case GPRUNE_SEL_BEAM:
-    compute_gaussset = gprune_beam;
-    compute_gaussset_init = gprune_beam_init;
-    compute_gaussset_free = gprune_beam_free;
+    wrk->compute_gaussset = gprune_beam;
+    wrk->compute_gaussset_init = gprune_beam_init;
+    wrk->compute_gaussset_free = gprune_beam_free;
     break;
   }
   /* select caching function to compute output probability of a mixture */
   if (hmminfo->is_tied_mixture) {
-    calc_outprob = calc_tied_mix; /* enable book-level cache, typically for a tied-mixture model */
+    wrk->calc_outprob = calc_tied_mix; /* enable book-level cache, typically for a tied-mixture model */
   } else {
-    calc_outprob = calc_mix; /* no mixture-level cache, for a shared-state, non tied-mixture model */
+    wrk->calc_outprob = calc_mix; /* no mixture-level cache, for a shared-state, non tied-mixture model */
   }
   
   /* select back-off functon for state probability calculation */
   if (gshmm != NULL) {
-    calc_outprob_state = gms_state; /* enable GMS */
+    wrk->calc_outprob_state = gms_state; /* enable GMS */
   } else {
-    calc_outprob_state = calc_outprob; /* call mixture outprob directly */
+    wrk->calc_outprob_state = wrk->calc_outprob; /* call mixture outprob directly */
   }
 
   /* store common variable to global */
-  OP_hmminfo = hmminfo;
-  OP_gshmm = gshmm;		/* NULL if GMS not used */
-  OP_gprune_num = gprune_mixnum;
+  wrk->OP_hmminfo = hmminfo;
+  wrk->OP_gshmm = gshmm;		/* NULL if GMS not used */
+  wrk->OP_gprune_num = gprune_mixnum;
 
   /* generate addlog table */
   make_log_tbl();
   
   /* initialize work area for mixture component pruning function */
-  if (compute_gaussset_init() == FALSE) return FALSE; /* OP_gprune may change */
+  if ((*(wrk->compute_gaussset_init))(wrk) == FALSE) return FALSE; /* OP_gprune may change */
   /* initialize work area for book level cache on tied-mixture model */
   if (hmminfo->is_tied_mixture) {
-    if (calc_tied_mix_init() == FALSE) return FALSE;
+    if (calc_tied_mix_init(wrk) == FALSE) return FALSE;
   }
   /* initialize work area for GMS */
-  if (OP_gshmm != NULL) {
-    if (gms_init(gms_num) == FALSE) return FALSE;
+  if (wrk->OP_gshmm != NULL) {
+    wrk->my_nbest = gms_num;
+    if (gms_init(wrk) == FALSE) return FALSE;
   }
   /* initialize cache for all output probabilities */
-  if (outprob_cache_init() == FALSE)  return FALSE;
+  if (outprob_cache_init(wrk) == FALSE)  return FALSE;
 
   /* initialize word area for computation of pseudo HMM set when N-max is specified */
   if (hmminfo->cdset_method == IWCD_NBEST) {
-    outprob_cd_nbest_init(hmminfo->cdmax_num);
+    outprob_cd_nbest_init(wrk, hmminfo->cdmax_num);
   }
 
   return TRUE;
@@ -149,43 +156,46 @@ outprob_init(HTK_HMM_INFO *hmminfo,
 
 /** 
  * Prepare for the next input of given frame length.
- * 
+ *
+ * @param wrk [i/o] HMM computation work area
  * @param framenum [in] input length in frame.
  * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-outprob_prepare(int framenum)
+outprob_prepare(HMMWork *wrk, int framenum)
 {
-  if (outprob_cache_prepare() == FALSE) return FALSE;
-  if (OP_gshmm != NULL) {
-    if (gms_prepare(framenum) == FALSE) return FALSE;
+  if (outprob_cache_prepare(wrk) == FALSE) return FALSE;
+  if (wrk->OP_gshmm != NULL) {
+    if (gms_prepare(wrk, framenum) == FALSE) return FALSE;
   }
-  if (OP_hmminfo->is_tied_mixture) {
-    if (calc_tied_mix_prepare(framenum) == FALSE) return FALSE;
+  if (wrk->OP_hmminfo->is_tied_mixture) {
+    if (calc_tied_mix_prepare(wrk, framenum) == FALSE) return FALSE;
   }
   /* reset last time */
-  OP_last_time = OP_time = -1;
+  wrk->OP_last_time = wrk->OP_time = -1;
   return TRUE;
 }
 
 /**
  * Free all work area for outprob computation.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  */
 void
-outprob_free()
+outprob_free(HMMWork *wrk)
 {
-  compute_gaussset_free();
-  if (OP_hmminfo->is_tied_mixture) {
-    calc_tied_mix_free();
+  (*(wrk->compute_gaussset_free))(wrk);
+  if (wrk->OP_hmminfo->is_tied_mixture) {
+    calc_tied_mix_free(wrk);
   }
-  if (OP_gshmm != NULL) {
-    gms_free();
+  if (wrk->OP_gshmm != NULL) {
+    gms_free(wrk);
   }
-  outprob_cache_free();
-  if (OP_hmminfo->cdset_method == IWCD_NBEST) {
-    outprob_cd_nbest_free();
+  outprob_cache_free(wrk);
+  if (wrk->OP_hmminfo->cdset_method == IWCD_NBEST) {
+    outprob_cd_nbest_free(wrk);
   }
 
 }
