@@ -1,7 +1,5 @@
 /**
  * @file   aptree.c
- * @author Akinobu LEE
- * @date   Thu Feb 17 15:21:53 2005
  * 
  * <JA>
  * @brief  ѥȥꥷڤѤ̾ǡݥ󥿤ξ
@@ -11,13 +9,16 @@
  * @brief  Patricia index tree for name lookup: data type = pointer
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 15:21:53 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -42,34 +43,6 @@ new_node()
   return(tmp);
 }
 
-/* temporary work area for search */
-static char *tmp_str;		///< Local work area: current string we are searching for
-static int maxbitplace;		///< Local work area: Maximum bit length of @a tmp_str
-
-/** 
- * Local bit test function for search.
- * 
- * @param bitplace [in] bit place to test.
- * 
- * @return the content of tested bit in @a tmp_str, either 0 or 1.
- */
-static int
-testbit_local(int bitplace)
-{
-  int maskptr;
-  int bitshift;
-  unsigned char maskbit;
-
-  if (bitplace >= maxbitplace) return 0;
-  maskptr = bitplace / 8;
-  if ((bitshift = bitplace % 8) != 0) {
-    maskbit = 0x80 >> bitshift;
-  } else {
-    maskbit = 0x80;
-  }
-  return(tmp_str[maskptr] & maskbit);
-}
-
 /** 
  * Recursive function to search the data in the tree
  * 
@@ -78,15 +51,15 @@ testbit_local(int bitplace)
  * @return pointer to the found data
  */
 static void *
-aptree_search_data_r(APATNODE *node)
+aptree_search_data_r(APATNODE *node, char *str, int maxbitplace)
 {
   if (node->left0 == NULL && node->right1 == NULL) {
     return(node->value.data);
   } else {
-    if (testbit_local(node->value.thres_bit) != 0) {
-      return(aptree_search_data_r(node->right1));
+    if (testbit_max(str, node->value.thres_bit, maxbitplace) != 0) {
+      return(aptree_search_data_r(node->right1, str, maxbitplace));
     } else {
-      return(aptree_search_data_r(node->left0));
+      return(aptree_search_data_r(node->left0, str, maxbitplace));
     }
   }
 }
@@ -103,11 +76,10 @@ void *
 aptree_search_data(char *str, APATNODE *node)
 {
   if (node == NULL) {
-    j_error("Error: aptree_search_data: no node, search for \"%s\" failed\n", str);
+    //("Error: aptree_search_data: no node, search for \"%s\" failed\n", str);
+    return NULL;
   }
-  tmp_str = str;
-  maxbitplace = strlen(str) * 8 + 8;
-  return(aptree_search_data_r(node));
+  return(aptree_search_data_r(node, str, strlen(str) * 8 + 8));
 }
 
 
@@ -195,8 +167,6 @@ aptree_add_entry(char *str, void *data, char *matchstr, APATNODE **rootnode)
 
 /*******************************************************************/
 
-static APATNODE *tmproot;	///< Work area to hold current root node for removing entry
-
 /** 
  * Recursive sunction to find and remove an entry.
  * 
@@ -205,7 +175,7 @@ static APATNODE *tmproot;	///< Work area to hold current root node for removing
  * @param up2 [in] parent parent node
  */
 static void
-aptree_remove_entry_r(APATNODE *now, APATNODE *up, APATNODE *up2)
+aptree_remove_entry_r(APATNODE *now, APATNODE *up, APATNODE *up2, char *str, int maxbitplace, APATNODE **root)
 {
   APATNODE *b;
 
@@ -215,14 +185,14 @@ aptree_remove_entry_r(APATNODE *now, APATNODE *up, APATNODE *up2)
     /* execute removal */
     if (up == NULL) {
       free(now);
-      tmproot = NULL;
+      *root = NULL;
       return;
     }
     b = (up->right1 == now) ? up->left0 : up->right1;
     if (up2 == NULL) {
       free(now);
       free(up);
-      tmproot = b;
+      *root = b;
       return;
     }
     if (up2->left0 == up) {
@@ -235,10 +205,10 @@ aptree_remove_entry_r(APATNODE *now, APATNODE *up, APATNODE *up2)
     return;
   } else {
     /* traverse */
-    if (testbit_local(now->value.thres_bit) != 0) {
-      aptree_remove_entry_r(now->right1, now, up);
+    if (testbit_max(str, now->value.thres_bit, maxbitplace) != 0) {
+      aptree_remove_entry_r(now->right1, now, up, str, maxbitplace, root);
     } else {
-      aptree_remove_entry_r(now->left0, now, up);
+      aptree_remove_entry_r(now->left0, now, up, str, maxbitplace, root);
     }
   }
 }
@@ -254,13 +224,10 @@ void
 aptree_remove_entry(char *str, APATNODE **rootnode)
 {
   if (*rootnode == NULL) {
-    j_error("Error: aptree_remove_entry: no node, deletion for \"%s\" failed\n", str);
+    jlog("Warning: aptree: no node, deletion for \"%s\" failed\n", str);
+    return;
   }
-  tmproot = *rootnode;
-  tmp_str = str;
-  maxbitplace = strlen(str) * 8 + 8;
-  aptree_remove_entry_r(tmproot, NULL, NULL);
-  *rootnode = tmproot;
+  aptree_remove_entry_r(*rootnode, NULL, NULL, str, strlen(str)*8+8, rootnode);
 }
 
 /*******************************************************************/
