@@ -1,7 +1,5 @@
 /**
  * @file   mymalloc.c
- * @author Akinobu LEE
- * @date   Thu Feb 17 16:27:03 2005
  * 
  * <JA>
  * @brief  ưŪݤԤؿ
@@ -16,13 +14,16 @@
  * exit immediately.
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 16:27:03 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -41,7 +42,8 @@ mymalloc(int size)
 {
   void *p;
   if ( (p = malloc(size)) == NULL) {
-    j_error("cannot malloc %d byte\n",size);
+    jlog("Error: mymalloc: failed to allocate %d bytes\n",size);
+    exit(1);
   }
   return p;
 }
@@ -59,7 +61,8 @@ myrealloc(void *ptr, int size)
 {
   void *p;
   if ( (p = realloc(ptr,size)) == NULL) {
-    j_error("cannot realloc %d byte\n",size);
+    jlog("Error: mymalloc: failed to reallocate %d bytes\n",size);
+    exit(1);
   }
   return p;
 }
@@ -77,7 +80,8 @@ mycalloc(int nelem, int elsize)
 {
   void *p;
   if ( (p = calloc(nelem,elsize)) == NULL) {
-    j_error("cannot calloc %d x %d byte\n",nelem, elsize);
+    jlog("Error: mymalloc: failed to clear-allocate %d bytes\n", nelem*elsize);
+    exit(1);
   }
   return p;
 }
