@@ -1,7 +1,5 @@
 /**
  * @file   voca_load_htkdict.c
- * @author Akinobu LEE
- * @date   Fri Feb 18 19:43:06 2005
  * 
  * <JA>
  * @brief  HTKñ켭ǡɤ߹
@@ -18,13 +16,16 @@
  * triphone will be done here.
  * </EN>
  * 
- * $Revision: 1.2 $
+ * @author Akinobu LEE
+ * @date   Fri Feb 18 19:43:06 2005
+ *
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -57,7 +58,7 @@
  *                 to express the pronunciation
  */
 
-#define PHONEMELEN_STEP  10	///< Memory allocation step for phoneme sequence
+#define PHONEMELEN_STEP  30	///< Memory allocation step for phoneme sequence
 static char buf[MAXLINELEN];	///< Local work area for input text processing
 static char bufbak[MAXLINELEN];	///< Local work area for debug message
 
@@ -145,7 +146,7 @@ add_to_error(WORD_INFO *winfo, char *name)
     winfo->errph_root = aptree_make_root_node(buf);
   } else {
     match = aptree_search_data(buf, winfo->errph_root);
-    if (!strmatch(match, buf)) {
+    if (match == NULL || !strmatch(match, buf)) {
       aptree_add_entry(buf, buf, match, &(winfo->errph_root));
     }
   }
@@ -161,7 +162,7 @@ callback_list_error(void *x)
 {
   char *name;
   name = x;
-  j_printf("%s\n", name);
+  jlog("Error: voca_load_htkdict: %s\n", name);
 }
 /** 
  * Output all error phones appeared while readin a word dictionary.
@@ -171,9 +172,9 @@ callback_list_error(void *x)
 static void
 list_error(WORD_INFO *winfo)
 {
-  j_printf("////// Missing phones:\n");
+  jlog("Error: voca_load_htkdict: begin missing phones\n");
   aptree_traverse_and_do(winfo->errph_root, callback_list_error);
-  j_printf("//////////////////////\n");
+  jlog("Error: voca_load_htkdict: end missing phones\n");
 }
 
 /** 
@@ -181,40 +182,115 @@ list_error(WORD_INFO *winfo)
  * 
  * @param winfo [i/o]
  */
-static void
-set_maxwn(WORD_INFO *winfo)
+void
+voca_set_stats(WORD_INFO *winfo)
 {
   int w,p,n;
   int maxwn;
+  int maxwlen;
+  int states;
+  int models;
+  int trnum;
 
   maxwn = 0;
+  maxwlen = 0;
+  states = 0;
+  models = 0;
+  trnum = 0;
   for (w=0;w<winfo->num;w++) {
+    models += winfo->wlen[w];
+    if (maxwlen < winfo->wlen[w]) maxwlen = winfo->wlen[w];
     n = 0;
     for (p=0;p<winfo->wlen[w];p++) {
       n += hmm_logical_state_num(winfo->wseq[w][p]) - 2;
     }
     if (maxwn < n) maxwn = n;
+    states += n;
+    if (winfo->is_transparent[w]) trnum++;
   }
   winfo->maxwn = maxwn;
+  winfo->maxwlen = maxwlen;
+  winfo->totalstatenum = states;
+  winfo->totalmodelnum = models;
+  winfo->totaltransnum = trnum;
 }
 
 /** 
- * Parse the word dictionary to set the maximum word length.
+ * Start loading a dictionary.  See voca_load_htkdict() for an example
+ * of using this function.
+ * 
+ * @param winfo [i/o] dictionary data where the data will be loaded
+ * @param hmminfo [in] phoneme HMM definition 
+ * @param ignore_tri_conv [in] if TRUE, skip triphone conversion while loading
  * 
- * @param winfo [i/o] word dictionary data
  */
-static void
-set_maxwlen(WORD_INFO *winfo)
+void
+voca_load_start(WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean ignore_tri_conv)
 {
-  WORD_ID w;
-  int maxwlen;
-  maxwlen = 0;
-  for(w=0;w<winfo->num;w++) {
-    if (maxwlen < winfo->wlen[w]) maxwlen = winfo->wlen[w];
+  winfo->ok_flag = TRUE;
+  winfo->linenum = 0;
+  if (hmminfo != NULL && hmminfo->is_triphone && (! ignore_tri_conv)) {
+    winfo->do_conv = TRUE;
+  } else {
+    winfo->do_conv = FALSE;
   }
-  winfo->maxwlen = maxwlen;
+  winfo_init(winfo);
+  winfo->num = 0;
 }
 
+/** 
+ * Load a line from buffer and set parameters to the dictionary.
+ * See voca_load_htkdict() for an example of using this function.
+ * 
+ * @param buf [in] input buffer containing a word entry
+ * @param winfo [i/o] word dictionary to append the entry
+ * @param hmminfo [in] phoneme HMM definition
+ * 
+ * @return TRUE when successfully read, or FALSE on encountered end of
+ * dictionary.  When an error occurs, this function will set winfo->ok_flag
+ * to FALSE.
+ * 
+ */
+boolean
+voca_load_line(char *buf, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo)
+{
+  WORD_ID vnum;
+
+  winfo->linenum++;
+  vnum = winfo->num;
+  if (vnum >= winfo->maxnum) {
+    if (winfo_expand(winfo) == FALSE) return FALSE;
+  }
+  if (voca_load_htkdict_line(buf, &vnum, winfo->linenum, winfo, hmminfo, winfo->do_conv, &(winfo->ok_flag)) == FALSE) {
+    return FALSE;
+  }
+  winfo->num = vnum;
+  return TRUE;
+}
+
+/** 
+ * End loading dictionary entries.  It calculates some statistics for
+ * the read entries, outputs errors if encountered
+ * while the last loading, and returns with status whether an error
+ * occured while loading.
+ * 
+ * @param winfo [i/o] word dictionary just read by voca_load_line() calls
+ * 
+ * @return TRUE when no error has been occured during loading, or FALSE
+ * if an error occured.
+ * 
+ */
+boolean
+voca_load_end(WORD_INFO *winfo)
+{
+  voca_set_stats(winfo);
+  if (!winfo->ok_flag) {
+    if (winfo->errph_root != NULL) list_error(winfo);
+  }
+  return(winfo->ok_flag);
+}
+
+
 /** 
  * Top function to read word dictionary via file pointer
  * 
@@ -228,30 +304,15 @@ set_maxwlen(WORD_INFO *winfo)
 boolean
 voca_load_htkdict(FILE *fp, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean ignore_tri_conv)
 {
-  boolean ok_flag = TRUE;
-  WORD_ID vnum;
-  boolean do_conv = FALSE;
+  boolean ret;
 
-  if (hmminfo != NULL && hmminfo->is_triphone && (! ignore_tri_conv))
-    do_conv = TRUE;
-
-  winfo_init(winfo);
-
-  vnum = 0;
+  voca_load_start(winfo, hmminfo, ignore_tri_conv);
   while (getl(buf, sizeof(buf), fp) != NULL) {
-    if (vnum >= winfo->maxnum) winfo_expand(winfo);
-    if (voca_load_htkdict_line(buf, vnum, winfo, hmminfo, do_conv, &ok_flag) == FALSE) break;
-    vnum++;
+    if (voca_load_line(buf, winfo, hmminfo) == FALSE) break;
   }
-  winfo->num = vnum;
+  ret = voca_load_end(winfo);
 
-  if (winfo->errph_root != NULL) list_error(winfo);
-
-  /* compute maxwn */
-  set_maxwn(winfo);
-  set_maxwlen(winfo);
-
-  return(ok_flag);
+  return(ret);
 }
 
 
@@ -268,30 +329,15 @@ voca_load_htkdict(FILE *fp, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean ign
 boolean
 voca_load_htkdict_fd(int fd, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean ignore_tri_conv)
 {
-  boolean ok_flag = TRUE;
-  WORD_ID vnum;
-  boolean do_conv = FALSE;
+  boolean ret;
 
-  if (hmminfo != NULL && hmminfo->is_triphone && (! ignore_tri_conv))
-    do_conv = TRUE;
-
-  winfo_init(winfo);
-
-  vnum = 0;
+  voca_load_start(winfo, hmminfo, ignore_tri_conv);
   while(getl_fd(buf, MAXLINELEN, fd) != NULL) {
-    if (vnum >= winfo->maxnum) winfo_expand(winfo);
-    if (voca_load_htkdict_line(buf, vnum, winfo, hmminfo, do_conv, &ok_flag) == FALSE) break;
-    vnum++;
+    if (voca_load_line(buf, winfo, hmminfo) == FALSE) break;
   }
-  winfo->num = vnum;
+  ret = voca_load_end(winfo);
 
-  if (winfo->errph_root != NULL) list_error(winfo);
-
-  /* compute maxwn */
-  set_maxwn(winfo);
-  set_maxwlen(winfo);
-
-  return(ok_flag);
+  return(ret);
 }
 
 /** 
@@ -307,30 +353,15 @@ voca_load_htkdict_fd(int fd, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean ig
 boolean
 voca_load_htkdict_sd(int sd, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean ignore_tri_conv)
 {
-  boolean ok_flag = TRUE;
-  WORD_ID vnum;
-  boolean do_conv = FALSE;
-  
-  if (hmminfo != NULL && hmminfo->is_triphone && (! ignore_tri_conv))
-    do_conv = TRUE;
-  
-  winfo_init(winfo);
-  
-  vnum = 0;
-  while(getl_sd(buf, MAXLINELEN, sd) != NULL) {
-    if (vnum >= winfo->maxnum) winfo_expand(winfo);
-    if (voca_load_htkdict_line(buf, vnum, winfo, hmminfo, do_conv, &ok_flag) == FALSE) break;
-    vnum++;
-  }
-  winfo->num = vnum;
-  
-  if (winfo->errph_root != NULL) list_error(winfo);
+  boolean ret;
 
-  /* compute maxwn */
-  set_maxwn(winfo);
-  set_maxwlen(winfo);
-  
-  return(ok_flag);
+  voca_load_start(winfo, hmminfo, ignore_tri_conv);
+  while(getl_sd(buf, MAXLINELEN, sd) != NULL) {
+    if (voca_load_line(buf, winfo, hmminfo) == FALSE) break;
+  }
+  ret = voca_load_end(winfo);
+
+  return(ret);
 }
 
 /** 
@@ -346,33 +377,16 @@ voca_load_htkdict_sd(int sd, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean ig
 boolean
 voca_append_htkdict(char *entry, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean ignore_tri_conv)
 {
-  boolean ok_flag = TRUE;
-  boolean do_conv = FALSE;
-
-  if (hmminfo != NULL && hmminfo->is_triphone && (! ignore_tri_conv))
-    do_conv = TRUE;
-
-  if (winfo->num >= winfo->maxnum) winfo_expand(winfo);
-  strcpy(buf, entry);		/* const buffer not allowed in voca_load_htkdict_line() */
-  voca_load_htkdict_line(buf, winfo->num, winfo, hmminfo, do_conv, &ok_flag);
-
-  if (ok_flag == TRUE) {
-    winfo->num++;
-    /* re-compute maxwn */
-    set_maxwn(winfo);
-    set_maxwlen(winfo);
-  } else {
-    if (winfo->errph_root != NULL) list_error(winfo);
-  }
-
-  return(ok_flag);
+  voca_load_line(entry, winfo, hmminfo);
+  return(voca_load_end(winfo));
 }
 
 /** 
  * Sub function to Add a dictionary entry line to the word dictionary.
  * 
  * @param buf [i/o] buffer to hold the input string, will be modified in this function
- * @param vnum [in] current number of words in @a winfo
+ * @param vnum_p [in] current number of words in @a winfo
+ * @param linenum [in] current line number of the input
  * @param winfo [out] pointer to word dictionary to append the data.
  * @param hmminfo [in] HTK %HMM definition data.  if NULL, phonemes are ignored.
  * @param do_conv [in] TRUE if performing triphone conversion
@@ -381,35 +395,44 @@ voca_append_htkdict(char *entry, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolea
  * @return FALSE if buf == "DICEND", else TRUE will be returned.
  */
 boolean
-voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean do_conv, boolean *ok_flag)
+voca_load_htkdict_line(char *buf, WORD_ID *vnum_p, int linenum, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean do_conv, boolean *ok_flag)
 {
   char *ptmp, *lp = NULL, *p;
   static char cbuf[MAX_HMMNAME_LEN];
-  int tmpmaxlen, len;
-  HMM_Logical **tmpwseq;
+  static HMM_Logical **tmpwseq = NULL;
+  static int tmpmaxlen;
+  int len;
   HMM_Logical *tmplg;
   boolean pok;
+  int vnum;
+
+  vnum = *vnum_p;
 
   if (strmatch(buf, "DICEND")) return FALSE;
 
+  /* allocate temporal work area for the first call */
+  if (tmpwseq == NULL) {
+    tmpmaxlen = PHONEMELEN_STEP;
+    tmpwseq = (HMM_Logical **)mymalloc(sizeof(HMM_Logical *) * tmpmaxlen);
+  }
+
   /* backup whole line for debug output */
   strcpy(bufbak, buf);
   
   /* GrammarEntry */
-  if ((ptmp = mystrtok(buf, " \t\n")) == NULL) {
-    j_printerr("line %d: corrupted data:\n> %s\n", vnum+1, bufbak);
+  if ((ptmp = mystrtok_quote(buf, " \t\n")) == NULL) {
+    jlog("Error: voca_load_htkdict: line %d: corrupted data:\n> %s\n", linenum, bufbak);
     winfo->errnum++;
     *ok_flag = FALSE;
     return TRUE;
   }
-  winfo->wname[vnum] = strcpy((char *)mymalloc(strlen(ptmp)+1), ptmp);
+  winfo->wname[vnum] = strcpy((char *)mybmalloc2(strlen(ptmp)+1, &(winfo->mroot)), ptmp);
 
   /* just move pointer to next token */
   if ((ptmp = mystrtok_movetonext(NULL, " \t\n")) == NULL) {
-    j_printerr("line %d: corrupted data:\n> %s\n", vnum+1, bufbak);
+    jlog("Error: voca_load_htkdict: line %d: corrupted data:\n> %s\n", linenum, bufbak);
     winfo->errnum++;
     *ok_flag = FALSE;
-    free(winfo->wname[vnum]);
     return TRUE;
   }
 #ifdef CLASS_NGRAM
@@ -424,41 +447,37 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
     /* format: @%f (log scale) */
     /* if "@" not found or "@0", it means class == word */
     if ((ptmp = mystrtok(NULL, " \t\n")) == NULL) {
-      j_printerr("line %d: corrupted data:\n> %s\n", vnum+1, bufbak);
+      jlog("Error: voca_load_htkdict: line %d: corrupted data:\n> %s\n", linenum, bufbak);
       winfo->errnum++;
       *ok_flag = FALSE;
       return TRUE;
     }
     if (ptmp[1] == '\0') {	/* space between '@' and figures */
-      j_printerr("line %d: value after '@' missing, maybe wrong space?\n> %s\n", vnum+1, bufbak);
+      jlog("Error: voca_load_htkdict: line %d: value after '@' missing, maybe wrong space?\n> %s\n", linenum, bufbak);
       winfo->errnum++;
       *ok_flag = FALSE;
-      free(winfo->wname[vnum]);
       return TRUE;
     }
     winfo->cprob[vnum] = atof(&(ptmp[1]));
     if (winfo->cprob[vnum] != 0.0) winfo->cwnum++;
     /* read next word entry (just skip them) */
     if ((ptmp = mystrtok(NULL, " \t\n")) == NULL) {
-      j_printerr("line %d: corrupted data:\n> %s\n", vnum+1,bufbak);
+      jlog("Error: voca_load_htkdict: line %d: corrupted data:\n> %s\n", linenum,bufbak);
       winfo->errnum++;
       *ok_flag = FALSE;
-      free(winfo->wname[vnum]);
       return TRUE;
     }
     /* move to the next word entry */
     if ((ptmp = mystrtok_movetonext(NULL, " \t\n")) == NULL) {
-      j_printerr("line %d: corrupted data:\n> %s\n", vnum+1, bufbak);
+      jlog("Error: voca_load_htkdict: line %d: corrupted data:\n> %s\n", linenum, bufbak);
       winfo->errnum++;
       *ok_flag = FALSE;
-      free(winfo->wname[vnum]);
       return TRUE;
     }
 #else  /* ~CLASS_NGRAM */
-    j_printerr("line %d: cannot handle in-class word probability\n> %s\n", vnum+1, ptmp, bufbak);
+    jlog("Error: voca_load_htkdict: line %d: cannot handle in-class word probability\n> %s\n", linenum, ptmp, bufbak);
     winfo->errnum++;
     *ok_flag = FALSE;
-    free(winfo->wname[vnum]);
     return TRUE;
 #endif /* CLASS_NGRAM */
   }
@@ -474,20 +493,26 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
     ptmp = mystrtok_quotation(NULL, " \t\n", '{', '}', 0);
     break;
   default:
-    j_printerr("line %d: missing output string??\n> %s\n", vnum+1, bufbak);
+#if 1
+    /* ALLOW no entry for output */
+    /* same as wname is used */
+    winfo->is_transparent[vnum] = FALSE;
+    ptmp = winfo->wname[vnum];
+#else
+    /* error */
+    jlog("Error: voca_load_htkdict: line %d: missing output string??\n> %s\n", linenum, bufbak);
     winfo->errnum++;
     *ok_flag = FALSE;
-    free(winfo->wname[vnum]);
     return TRUE;
+#endif
   }
   if (ptmp == NULL) {
-    j_printerr("line %d: corrupted data:\n> %s\n", vnum+1, bufbak);
+    jlog("Error: voca_load_htkdict: line %d: corrupted data:\n> %s\n", linenum, bufbak);
     winfo->errnum++;
     *ok_flag = FALSE;
-    free(winfo->wname[vnum]);
     return TRUE;
   }
-  winfo->woutput[vnum] = strcpy((char *)mymalloc(strlen(ptmp)+1), ptmp);
+  winfo->woutput[vnum] = strcpy((char *)mybmalloc2(strlen(ptmp)+1, &(winfo->mroot)), ptmp);
     
   /* phoneme sequence */
   if (hmminfo == NULL) {
@@ -495,19 +520,17 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
     winfo->wseq[vnum] = NULL;
     winfo->wlen[vnum] = 0;
   } else {
-    tmpmaxlen = PHONEMELEN_STEP;
-    tmpwseq = (HMM_Logical **)mymalloc(sizeof(HMM_Logical *) * tmpmaxlen);
+
+    /* store converted phone sequence to temporal bufffer */
     len = 0;
       
     if (do_conv) {
       /* convert phoneme to triphone expression (word-internal) */
       cycle_triphone(NULL);
       if ((lp = mystrtok(NULL, " \t\n")) == NULL) {
-	j_printerr("line %d: word %s has no phoneme:\n> %s\n", vnum+1, winfo->wname[vnum], bufbak);
+	jlog("Error: voca_load_htkdict: line %d: word %s has no phoneme:\n> %s\n", linenum, winfo->wname[vnum], bufbak);
 	winfo->errnum++;
 	*ok_flag = FALSE;
-	free(winfo->wname[vnum]);
-	free(winfo->woutput[vnum]);
 	return TRUE;
       }
       cycle_triphone(lp);
@@ -516,7 +539,7 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
     pok = TRUE;
     for (;;) {
       if (do_conv) {
-/*	if (lp != NULL) j_printf(" %d%s",len,lp);*/
+/*	if (lp != NULL) jlog(" %d%s",len,lp);*/
 	if (lp != NULL) lp = mystrtok(NULL, " \t\n");
 	if (lp != NULL) p = cycle_triphone(lp);
 	else p = cycle_triphone_flush();
@@ -532,20 +555,20 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
 	if (do_conv) {
 	  /* both defined or pseudo phone are not found */
 	  if (len == 0 && lp == NULL) {
-	    j_printerr("line %d: triphone \"*-%s+*\" or monophone \"%s\" not found\n", vnum+1, p, p);
+	    jlog("Error: voca_load_htkdict: line %d: triphone \"*-%s+*\" or monophone \"%s\" not found\n", linenum, p, p);
 	    snprintf(cbuf,MAX_HMMNAME_LEN,"*-%s+* or monophone %s", p, p);
 	  } else if (len == 0) {
-	    j_printerr("line %d: triphone \"*-%s\" or biphone \"%s\" not found\n", vnum+1, p, p);
+	    jlog("Error: voca_load_htkdict: line %d: triphone \"*-%s\" or biphone \"%s\" not found\n", linenum, p, p);
 	    snprintf(cbuf,MAX_HMMNAME_LEN,"*-%s or biphone %s", p, p);
 	  } else if (lp == NULL) {
-	    j_printerr("line %d: triphone \"%s+*\" or biphone \"%s\" not found\n", vnum+1, p, p);
+	    jlog("Error: voca_load_htkdict: line %d: triphone \"%s+*\" or biphone \"%s\" not found\n", linenum, p, p);
 	    snprintf(cbuf,MAX_HMMNAME_LEN,"%s+* or biphone %s", p, p);
 	  } else {
-	    j_printerr("line %d: triphone \"%s\" not found\n", vnum+1, p);
+	    jlog("Error: voca_load_htkdict: line %d: triphone \"%s\" not found\n", linenum, p);
 	    snprintf(cbuf,MAX_HMMNAME_LEN,"%s", p);
 	  }
 	} else {
-	  j_printerr("line %d: phone \"%s\" not found\n", vnum+1, p);
+	  jlog("Error: voca_load_htkdict: line %d: phone \"%s\" not found\n", linenum, p);
 	  snprintf(cbuf, MAX_HMMNAME_LEN, "%s", p);
 	}
 	add_to_error(winfo, cbuf);
@@ -557,29 +580,32 @@ voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmmi
 	  tmpmaxlen += PHONEMELEN_STEP;
 	  tmpwseq = (HMM_Logical **)myrealloc(tmpwseq, sizeof(HMM_Logical *) * tmpmaxlen);
 	}
+	/* store to temporal buffer */
 	tmpwseq[len] = tmplg;
       }
       len++;
     }
     if (!pok) {			/* error in phoneme */
-      j_printerr("> %s\n", bufbak);
+      jlog("Error: voca_load_htkdict: the line content was: %s\n", bufbak);
       winfo->errnum++;
       *ok_flag = FALSE;
-      free(winfo->wname[vnum]);
-      free(winfo->woutput[vnum]);
       return TRUE;
     }
     if (len == 0) {
-      j_printerr("line %d: no phone specified:\n> %s\n", vnum+1, bufbak);
+      jlog("Error: voca_load_htkdict: line %d: no phone specified:\n> %s\n", linenum, bufbak);
       winfo->errnum++;
       *ok_flag = FALSE;
-      free(winfo->wname[vnum]);
-      free(winfo->woutput[vnum]);
       return TRUE;
     }
-    winfo->wseq[vnum] = tmpwseq;
+    /* store to winfo */
+    winfo->wseq[vnum] = (HMM_Logical **)mybmalloc2(sizeof(HMM_Logical *) * len, &(winfo->mroot));
+    memcpy(winfo->wseq[vnum], tmpwseq, sizeof(HMM_Logical *) * len);
     winfo->wlen[vnum] = len;
   }
+
+  vnum++;
+
+  *vnum_p = vnum;
   
   return(TRUE);
 }
@@ -615,7 +641,7 @@ voca_mono2tri(WORD_INFO *winfo, HTK_HMM_INFO *hmminfo)
 	p = cycle_triphone(winfo->wseq[w][ph + 1]->name);
       }
       if ((tmplg = htk_hmmdata_lookup_logical(hmminfo, p)) == NULL) {
-	j_printerr("voca_mono2tri: word \"%s[%s]\"(id=%d): HMM \"%s\" not found\n", winfo->wname[w], winfo->woutput[w], w, p);
+	jlog("Error: voca_load_htkdict: word \"%s[%s]\"(id=%d): HMM \"%s\" not found\n", winfo->wname[w], winfo->woutput[w], w, p);
 	ok_flag = FALSE;
 	continue;
       }
@@ -636,7 +662,7 @@ voca_mono2tri(WORD_INFO *winfo, HTK_HMM_INFO *hmminfo)
  * @param woffset [in] word id offset in @a dst where the new data
  * should be stored
  */
-void
+boolean
 voca_append(WORD_INFO *dstinfo, WORD_INFO *srcinfo, int coffset, int woffset)
 {
   WORD_ID n, w;
@@ -646,9 +672,9 @@ voca_append(WORD_INFO *dstinfo, WORD_INFO *srcinfo, int coffset, int woffset)
   for(w=0;w<srcinfo->num;w++) {
     /* copy data */
     dstinfo->wlen[n] = srcinfo->wlen[w];
-    dstinfo->wname[n] = strcpy((char *)mymalloc(strlen(srcinfo->wname[w])+1), srcinfo->wname[w]);
-    dstinfo->woutput[n] = strcpy((char *)mymalloc(strlen(srcinfo->woutput[w])+1), srcinfo->woutput[w]);
-    dstinfo->wseq[n] = (HMM_Logical **)mymalloc(sizeof(HMM_Logical *) * srcinfo->wlen[w]);
+    if (srcinfo->wname[w]) dstinfo->wname[n] = strcpy((char *)mybmalloc2(strlen(srcinfo->wname[w])+1, &(dstinfo->mroot)), srcinfo->wname[w]);
+    if (srcinfo->woutput[w]) dstinfo->woutput[n] = strcpy((char *)mybmalloc2(strlen(srcinfo->woutput[w])+1, &(dstinfo->mroot)), srcinfo->woutput[w]);
+    if (srcinfo->wlen[w] > 0) dstinfo->wseq[n] = (HMM_Logical **)mybmalloc2(sizeof(HMM_Logical *) * srcinfo->wlen[w], &(dstinfo->mroot));
     for(i=0;i<srcinfo->wlen[w];i++) {
       dstinfo->wseq[n][i] = srcinfo->wseq[w][i];
     }
@@ -657,12 +683,15 @@ voca_append(WORD_INFO *dstinfo, WORD_INFO *srcinfo, int coffset, int woffset)
     dstinfo->wton[n] = srcinfo->wton[w] + coffset;
     
     n++;
-    if (n >= dstinfo->maxnum) winfo_expand(dstinfo);
+    if (n >= dstinfo->maxnum) {
+      if (winfo_expand(dstinfo) == FALSE) return FALSE;
+    }
   }
   dstinfo->num = n;
 
   /* compute maxwn */
-  set_maxwn(dstinfo);
-  set_maxwlen(dstinfo);
+  voca_set_stats(dstinfo);
+
+  return TRUE;
 }
 
