@@ -16,7 +16,7 @@
  * @author Akinobu Lee
  * @date   Fri Oct 27 14:55:00 2006
  *
- * $Revision: 1.2 $
+ * $Revision: 1.4 $
  * 
  */
 /*
@@ -59,6 +59,7 @@ undef_para(Value *para)
   //para->ss_alpha   = -1;
   //para->ss_floor   = -1;
   para->zmeanframe = -1;
+  para->usepower   = -1;
   para->delta      = -1;
   para->acc        = -1;
   para->energy     = -1;
@@ -97,6 +98,7 @@ make_default_para(Value *para)
   //para->ss_alpha    = DEF_SSALPHA;
   //para->ss_floor    = DEF_SSFLOOR;
   para->zmeanframe = FALSE;
+  para->usepower   = FALSE;
 }
 
 /** 
@@ -122,6 +124,7 @@ make_default_para_htk(Value *para)
   para->hipass     = -1;	/* disabled */
   para->lopass     = -1;	/* disabled */
   para->zmeanframe = FALSE;
+  para->usepower   = FALSE;
 }
 
 /** 
@@ -155,6 +158,7 @@ apply_para(Value *dst, Value *src)
   //if (dst->ss_alpha   == -1) dst->ss_alpha = src->ss_alpha; 
   //if (dst->ss_floor   == -1) dst->ss_floor = src->ss_floor; 
   if (dst->zmeanframe == -1) dst->zmeanframe = src->zmeanframe; 
+  if (dst->usepower   == -1) dst->usepower = src->usepower; 
   if (dst->delta      == -1) dst->delta = src->delta; 
   if (dst->acc        == -1) dst->acc = src->acc; 
   if (dst->energy     == -1) dst->energy = src->energy; 
@@ -213,6 +217,7 @@ htk_config_file_parse(char *HTKconffile, Value *para)
 
     /* process arguments */
     skipped = FALSE;
+    srate = 0.0;
     if (strmatch(d, "SOURCERATE")) { /* -smpPeriod */
       srate = atof(a);
     } else if (strmatch(d, "TARGETRATE")) { /* -fshift */
@@ -221,6 +226,8 @@ htk_config_file_parse(char *HTKconffile, Value *para)
       para->framesize = atof(a);
     } else if (strmatch(d, "ZMEANSOURCE")) { /* -zmeansource */
       para->zmeanframe = (a[0] == 'T') ? TRUE : FALSE;
+    } else if (strmatch(d, "USEPOWER")) { /* -usepower */
+      para->zmeanframe = (a[0] == 'T') ? TRUE : FALSE;
     } else if (strmatch(d, "PREEMCOEF")) { /* -preemph */
       para->preEmph = atof(a);
     } else if (strmatch(d, "USEHAMMING")) { /* (fixed to T) */
@@ -249,12 +256,13 @@ htk_config_file_parse(char *HTKconffile, Value *para)
     } else if (strmatch(d, "SILFLOOR")) { /* -silfloor */
       para->silFloor = atof(a);
     } else if (strmatch(d, "TARGETKIND")) {
-      jlog("Stat: para: TARGETKIND skipped (will be set from AM header)\n");
+      jlog("Warning: para: TARGETKIND skipped (will be determined by AM header)\n");
       skipped = TRUE;
     } else if (strmatch(d, "NUMCEPS")) {
-      jlog("Stat: para: NUMCEPS skipped (will be set from AM header)\n");
+      jlog("Warning: para: NUMCEPS skipped (will be determined by AM header)\n");
       skipped = TRUE;
     } else {
+      jlog("Warning: para: \"%s\" ignored (not supported, or irrelevant)\n", d);
       skipped = TRUE;
     }
     if (!skipped) {
@@ -262,6 +270,12 @@ htk_config_file_parse(char *HTKconffile, Value *para)
     }
   }
 
+  if (srate == 0.0) {
+    jlog("Warning: no SOURCERATE found\n");
+    jlog("Warning: assume source waveform sample rate to 625 (16kHz)\n");
+    srate = 625;
+  }
+
   para->smp_period = srate;
   para->smp_freq = period2freq(para->smp_period);
   para->frameshift /= srate;
@@ -362,4 +376,7 @@ put_para(FILE *fp, Value *para)
   fprintf(fp, "\t zero mean frame = ");
   if (para->zmeanframe) fprintf(fp, "ON\n");
   else fprintf(fp, "OFF\n");
+  fprintf(fp, "\t       use power = ");
+  if (para->usepower) fprintf(fp, "ON\n");
+  else fprintf(fp, "OFF\n");
 }
