@@ -1,7 +1,5 @@
 /**
  * @file   wav2mfcc-pipe.c
- * @author Akinobu LEE
- * @date   Thu Feb 17 18:12:30 2005
  * 
  * <JA>
  * @brief  ȷ MFCC ħ̤Ѵ (ե졼ñ)
@@ -19,13 +17,16 @@
  * these functions will be used instead of ones in wav2mfcc.c
  * </EN>
  * 
- * $Revision: 1.6 $
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 18:12:30 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -45,89 +46,6 @@
 #include <sent/stddefs.h>
 #include <sent/mfcc.h>
 
-static double *fbank;                    ///< Local buffer to hold filterbank
-static FBankInfo fb;		///< Local buffer to hold filterbank information
-
-/** 
- * initialize and setup buffers for a MFCC computataion.
- * 
- * @param para [in] configuration parameters
- * @param bf [out] pointer to the entry point of workspace for FFT
- * @param ssbuf [in] noise spectrum, or NULL if not using spectral subtraction
- * @param ssbuflen [in] length of above, ignoredwhen @a ssbuf is NULL
- */
-void
-WMP_init(Value para, float **bf, float *ssbuf, int ssbuflen)
-{
-  /* Get filterbank information and initialize tables */
-  fb = InitFBank(para);
-  
-  if((fbank = (double *)mymalloc((para.fbank_num+1)*sizeof(double))) == NULL){
-    j_error("WMP_init failed\n");
-  }
-  if((*bf = (float *)mymalloc(fb.fftN * sizeof(float))) == NULL){
-    j_error("WMP_init failed\n");
-  }
-
-  if (ssbuf != NULL) {
-    /* check ssbuf length */
-    if (fb.fftN != ssbuflen) {
-      j_error("Error: Wav2MFCC: noise spectrum length not match\n");
-    }
-  }
-
-  /* Normalise Log Energy is not implemented in pipeline mode */
-  if(para.enormal) {
-    j_error("Error: \"normalize log energy\" is not supported in live input mode\n");
-  }
-}
-
-/** 
- * Calculate MFCC and log energy for one frame.  Perform spectral subtraction
- * if @a ssbuf is specified.
- * 
- * @param mfcc [out] buffer to hold the resulting MFCC vector
- * @param bf [i/o] work area for FFT
- * @param para [in] configuration parameters
- * @param ssbuf [in] noise spectrum, or NULL if not using spectral subtraction
- */
-void
-WMP_calc(float *mfcc, float *bf, Value para, float *ssbuf)
-{
-  float energy = 0.0;
-  float c0 = 0.0;
-  int p;
-
-  if (para.zmeanframe) {
-    ZMeanFrame(bf, para.framesize);
-  }
-
-  if (para.energy && para.raw_e) {
-    /* calculate log raw energy */
-    energy = CalcLogRawE(bf, para.framesize);
-  }
-  /* pre-emphasize */
-  PreEmphasise(bf, para);
-  /* hamming window */
-  Hamming(bf, para.framesize);
-  if (para.energy && ! para.raw_e) {
-    /* calculate log energy */
-    energy = CalcLogRawE(bf, para.framesize);
-  }
-  /* filterbank */
-  MakeFBank(bf, fbank, fb, para, ssbuf);
-  /* 0'th cepstral parameter */
-  if (para.c0) c0 = CalcC0(fbank, para);
-  /* MFCC */
-  MakeMFCC(fbank, mfcc, para);
-  /* weight cepstrum */
-  WeightCepstrum(mfcc, para);
-  /* set energy to mfcc */
-  p = para.mfcc_dim;
-  if (para.c0) mfcc[p++] = c0;
-  if (para.energy) mfcc[p++] = energy;
-}
-
 /***********************************************************************/
 /** 
  * Allocate a new delta cycle buffer.
@@ -317,26 +235,6 @@ WMP_deltabuf_flush(DeltaBuf *db)
 /* MAP-CMN */
 /***********************************************************************/
 
-#define CPMAX 500		///< Maximum number of frames to store ceptral mean for CMN update
-
-/**
- * Hold sentence sum of MFCC
- * 
- */
-typedef struct {
-  float *mfcc_sum;		///< values of sum of MFCC parameters
-  int framenum;			///< summed number of frames
-} CMEAN;
-#define CPSTEP 5		///< clist allocate step
-static CMEAN *clist;		///< List of MFCC sum for previous inputs
-static int clist_max;		///< Allocated number of CMEAN in clist
-static int clist_num;		///< Currentlly filled CMEAN in clist
-static int dim;			///< Local workarea to store the number of MFCC dimension.
-static float cweight;		///< Weight of initial cepstral mean
-static float *cmean_init;	///< Initial cepstral mean for each input
-static boolean cmean_init_set;	///< TRUE if cmean_init was set
-static CMEAN now;		///< Work area to hold current cepstral mean
-
 /**
  * Initialize MAP-CMN at startup.
  * 
@@ -344,68 +242,94 @@ static CMEAN now;		///< Work area to hold current cepstral mean
  * @param weight [in] initial cepstral mean weight
  * 
  */
-void
-CMN_realtime_init(int dimension, float weight)
+CMNWork *
+CMN_realtime_new(int dimension, float weight)
 {
   int i;
 
-  dim = dimension;
-  cweight = weight;
+  CMNWork *c;
 
-  clist_max = CPSTEP;
-  clist_num = 0;
-  clist = (CMEAN *)mymalloc(sizeof(CMEAN) * clist_max);
-  for(i=0;i<clist_max;i++) {
-    clist[i].mfcc_sum = (float *)mymalloc(sizeof(float)*dim);
-    clist[i].framenum = 0;
+  c = (CMNWork *)mymalloc(sizeof(CMNWork));
+
+  c->dim = dimension;
+  c->cweight = weight;
+
+  c->clist_max = CPSTEP;
+  c->clist_num = 0;
+  c->clist = (CMEAN *)mymalloc(sizeof(CMEAN) * c->clist_max);
+  for(i=0;i<c->clist_max;i++) {
+    c->clist[i].mfcc_sum = (float *)mymalloc(sizeof(float)*c->dim);
+    c->clist[i].framenum = 0;
   }
+  c->now.mfcc_sum = (float *)mymalloc(sizeof(float) * c->dim);
 
-  now.mfcc_sum = (float *)mymalloc(sizeof(float) * dim);
+  c->cmean_init = (float *)mymalloc(sizeof(float) * c->dim);
+  c->cmean_init_set = FALSE;
 
-  cmean_init = (float *)mymalloc(sizeof(float) * dim);
-  cmean_init_set = FALSE;
+  return c;
+}
 
+/** 
+ * Free work area for real-time CMN.
+ * 
+ * @param c [i/o] CMN calculation work area
+ * 
+ */
+void
+CMN_realtime_free(CMNWork *c)
+{
+  int i;
+
+  free(c->cmean_init);
+  free(c->now.mfcc_sum);
+  for(i=0;i<c->clist_max;i++) {
+    free(c->clist[i].mfcc_sum);
+  }
+  free(c->clist);
+  free(c);
 }
 
 /**
  * Prepare for MAP-CMN at start of each input
  * 
+ * @param c [i/o] CMN calculation work area
  */
 void
-CMN_realtime_prepare()
+CMN_realtime_prepare(CMNWork *c)
 {
   int d;
-  for(d=0;d<dim;d++) now.mfcc_sum[d] = 0.0;
-  now.framenum = 0;
+  
+  for(d=0;d<c->dim;d++) c->now.mfcc_sum[d] = 0.0;
+  c->now.framenum = 0;
 }
 
 /**
  * Perform MAP-CMN for incoming MFCC vectors
  * 
+ * @param c [i/o] CMN calculation work area
  * @param mfcc [in] MFCC vector
- * @param dim [in] dimension
  * 
  */
 void
-CMN_realtime(float *mfcc, int dim)
+CMN_realtime(CMNWork *c, float *mfcc)
 {
   int d;
   double x, y;
 
-  now.framenum++;
-  if (cmean_init_set) {
-    for(d=0;d<dim;d++) {
+  c->now.framenum++;
+  if (c->cmean_init_set) {
+    for(d=0;d<c->dim;d++) {
       /* accumulate value of given MFCC to sum */
-      now.mfcc_sum[d] += mfcc[d];
+      c->now.mfcc_sum[d] += mfcc[d];
       /* calculate map-cmn and perform subtraction to the given vector */
-      x = now.mfcc_sum[d] + cweight * cmean_init[d];
-      y = (double)now.framenum + cweight;
+      x = c->now.mfcc_sum[d] + c->cweight * c->cmean_init[d];
+      y = (double)c->now.framenum + c->cweight;
       mfcc[d] -= x / y;
     }
   } else {
-    for(d=0;d<dim;d++) {
-      now.mfcc_sum[d] += mfcc[d];
-      mfcc[d] -= now.mfcc_sum[d] / now.framenum;
+    for(d=0;d<c->dim;d++) {
+      c->now.mfcc_sum[d] += mfcc[d];
+      mfcc[d] -= c->now.mfcc_sum[d] / c->now.framenum;
     }
   }
 }
@@ -413,9 +337,10 @@ CMN_realtime(float *mfcc, int dim)
 /**
  * Update initial cepstral mean from previous utterances for next input.
  * 
+ * @param c [i/o] CMN calculation work area
  */
 void
-CMN_realtime_update()
+CMN_realtime_update(CMNWork *c)
 {
   float *tmp;
   int i, d;
@@ -423,38 +348,38 @@ CMN_realtime_update()
 
   /* if CMN_realtime was never called before this, return immediately */
   /* this may occur by pausing just after startup */
-  if (now.framenum == 0) return;
+  if (c->now.framenum == 0) return;
 
   /* compute cepstral mean from now and previous sums up to CPMAX frames */
-  for(d=0;d<dim;d++) cmean_init[d] = now.mfcc_sum[d];
-  frames = now.framenum;
-  for(i=0;i<clist_num;i++) {
-    for(d=0;d<dim;d++) cmean_init[d] += clist[i].mfcc_sum[d];
-    frames += clist[i].framenum;
+  for(d=0;d<c->dim;d++) c->cmean_init[d] = c->now.mfcc_sum[d];
+  frames = c->now.framenum;
+  for(i=0;i<c->clist_num;i++) {
+    for(d=0;d<c->dim;d++) c->cmean_init[d] += c->clist[i].mfcc_sum[d];
+    frames += c->clist[i].framenum;
     if (frames >= CPMAX) break;
   }
-  for(d=0;d<dim;d++) cmean_init[d] /= (float) frames;
-  cmean_init_set = TRUE;
+  for(d=0;d<c->dim;d++) c->cmean_init[d] /= (float) frames;
+  c->cmean_init_set = TRUE;
 
   /* expand clist if neccessary */
-  if (clist_num == clist_max && frames < CPMAX) {
-    clist_max += CPSTEP;
-    clist = (CMEAN *)myrealloc(clist, sizeof(CMEAN) * clist_max);
-    for(i=clist_num;i<clist_max;i++) {
-      clist[i].mfcc_sum = (float *)mymalloc(sizeof(float)*dim);
-      clist[i].framenum = 0;
+  if (c->clist_num == c->clist_max && frames < CPMAX) {
+    c->clist_max += CPSTEP;
+    c->clist = (CMEAN *)myrealloc(c->clist, sizeof(CMEAN) * c->clist_max);
+    for(i=c->clist_num;i<c->clist_max;i++) {
+      c->clist[i].mfcc_sum = (float *)mymalloc(sizeof(float)*c->dim);
+      c->clist[i].framenum = 0;
     }
   }
   
   /* shift clist */
-  tmp = clist[clist_max-1].mfcc_sum;
-  memcpy(&(clist[1]), &(clist[0]), sizeof(CMEAN) * (clist_max - 1));
-  clist[0].mfcc_sum = tmp;
+  tmp = c->clist[c->clist_max-1].mfcc_sum;
+  memmove(&(c->clist[1]), &(c->clist[0]), sizeof(CMEAN) * (c->clist_max - 1));
+  c->clist[0].mfcc_sum = tmp;
   /* copy now to clist[0] */
-  memcpy(clist[0].mfcc_sum, now.mfcc_sum, sizeof(float) * dim);
-  clist[0].framenum = now.framenum;
+  memcpy(c->clist[0].mfcc_sum, c->now.mfcc_sum, sizeof(float) * c->dim);
+  c->clist[0].framenum = c->now.framenum;
 
-  if (clist_num < clist_max) clist_num++;
+  if (c->clist_num < c->clist_max) c->clist_num++;
 
 }
 
@@ -491,7 +416,7 @@ myread(void *buf, size_t unitbyte, int unitnum, FILE *fp)
  * @return TRUE if required number of units are fully written, FALSE if failed.
  */
 static boolean
-mywrite(void *buf, size_t unitbyte, int unitnum, int fd)
+mywrite(void *buf, size_t unitbyte, size_t unitnum, int fd)
 {
 #ifndef WORDS_BIGENDIAN
   swap_bytes(buf, unitbyte, unitnum);
@@ -509,44 +434,47 @@ mywrite(void *buf, size_t unitbyte, int unitnum, int fd)
  * Load CMN parameter from file.  If the number of MFCC dimension in the
  * file does not match the specified one, an error will occur.
  * 
+ * @param c [i/o] CMN calculation work area
  * @param filename [in] file name
- * @param dim [in] required number of MFCC dimensions
  * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-CMN_load_from_file(char *filename, int dim)
+CMN_load_from_file(CMNWork *c, char *filename)
 {
   FILE *fp;
   int veclen;
+
+  jlog("Stat: wav2mfcc-pipe: reading initial CMN from file \"%s\"\n", filename);
   if ((fp = fopen_readfile(filename)) == NULL) {
-    j_printerr("Error: CMN_load_from_file: failed to open\n");
+    jlog("Error: wav2mfcc-pipe: failed to open\n");
     return(FALSE);
   }
   /* read header */
   if (myread(&veclen, sizeof(int), 1, fp) == FALSE) {
-    j_printerr("Error: CMN_load_from_file: failed to read header\n");
+    jlog("Error: wav2mfcc-pipe: failed to read header\n");
     fclose_readfile(fp);
     return(FALSE);
   }
   /* check length */
-  if (veclen != dim) {
-    j_printerr("Error: CMN_load_from_file: vector dimension mismatch\n");
+  if (veclen != c->dim) {
+    jlog("Error: wav2mfcc-pipe: vector dimension mismatch\n");
     fclose_readfile(fp);
     return(FALSE);
   }
   /* read body */
-  if (myread(cmean_init, sizeof(float), dim, fp) == FALSE) {
-    j_printerr("Error: CMN_load_from_file: failed to read\n");
+  if (myread(c->cmean_init, sizeof(float), c->dim, fp) == FALSE) {
+    jlog("Error: wav2mfcc-pipe: failed to read\n");
     fclose_readfile(fp);
     return(FALSE);
   }
   if (fclose_readfile(fp) == -1) {
-    j_printerr("Error: CMN_load_from_file: failed to close\n");
+    jlog("Error: wav2mfcc-pipe: failed to close\n");
     return(FALSE);
   }
 
-  cmean_init_set = TRUE;
+  c->cmean_init_set = TRUE;
+  jlog("Stat: wav2mfcc-pipe: read CMN parameter\n");
 
   return(TRUE);
 }
@@ -554,32 +482,87 @@ CMN_load_from_file(char *filename, int dim)
 /** 
  * Save the current CMN vector to a file.
  * 
+ * @param c [i/o] CMN calculation work area
  * @param filename [in] filename to save the data.
  * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-CMN_save_to_file(char *filename)
+CMN_save_to_file(CMNWork *c, char *filename)
 {
   int fd;
 
+  jlog("Stat: wav2mfcc-pipe: writing current CM to file \"%s\"\n", filename);
+
   if ((fd = creat(filename, 0644)) == -1) {
-    j_printerr("Error: CMN_save_to_file: failed to open\n");
+    jlog("Error: wav2mfcc-pipe: failed to open\n");
     return(FALSE);
   }
   /* write header */
-  if (mywrite(&dim, sizeof(int), 1, fd) == FALSE) {
-    j_printerr("Error: CMN_save_to_file: failed to write header\n");
+  if (mywrite(&(c->dim), sizeof(int), 1, fd) == FALSE) {
+    jlog("Error: wav2mfcc-pipe: failed to write header\n");
     close(fd);
     return(FALSE);
   }
   /* write body */
-  if (mywrite(cmean_init, sizeof(float), dim, fd) == FALSE) {
-    j_printerr("Error: CMN_save_to_file: failed to write header\n");
+  if (mywrite(c->cmean_init, sizeof(float), c->dim, fd) == FALSE) {
+    jlog("Error: wav2mfcc-pipe: failed to write header\n");
     close(fd);
     return(FALSE);
   }
   close(fd);
+
+  jlog("Stat: wav2mfcc-pipe: wrote current CM\n");
   
   return(TRUE);
 }
+
+
+/***********************************************************************/
+/* energy normalization and scaling on live input */
+/***********************************************************************/
+
+/** 
+ * Initialize work area for energy normalization on live input.
+ * This should be called once on startup.
+ * 
+ * @param energy [in] energy normalization work area
+ * 
+ */
+void
+energy_max_init(ENERGYWork *energy)
+{
+  energy->max = 5.0;
+}
+
+/**
+ * Prepare values for energy normalization on live input.
+ * This should be called before each input segment.
+ * 
+ * @param energy [in] energy normalization work area
+ * @param para [in] MFCC computation configuration parameter
+ */
+void
+energy_max_prepare(ENERGYWork *energy, Value *para)
+{
+  energy->max_last = energy->max;
+  energy->min_last = energy->max - (para->silFloor * LOG_TEN) / 10.0;
+  energy->max = 0.0;
+}
+
+/** 
+ * Peform energy normalization using maximum of last input.
+ * 
+ * @param energy [in] energy normalization work area
+ * @param f [in] raw energy
+ * @param para [in] MFCC computation configuration parameter
+ * 
+ * @return value of the normalized log energy.
+ */
+LOGPROB
+energy_max_normalize(ENERGYWork *energy, LOGPROB f, Value *para)
+{
+  if (energy->max < f) energy->max = f;
+  if (f < energy->min_last) f = energy->min_last;
+  return(1.0 - (energy->max_last - f) * para->escale);
+}
